import os
import platform
import textwrap

import pytest

from conan.tools.cmake import CMakeToolchain
from conans.test.functional.toolchains.android._utils import create_library
from conans.test.utils.tools import TestClient


@pytest.fixture
def client():
    t = TestClient()
    create_library(t)
    t.save({
        'conanfile.py': textwrap.dedent("""
            from conans import ConanFile
            from conan.tools.cmake import CMake, CMakeToolchain

            class Library(ConanFile):
                name = 'library'
                settings = 'os', 'arch', 'compiler', 'build_type'
                exports_sources = "CMakeLists.txt", "lib.h", "lib.cpp"
                options = {'shared': [True, False]}
                default_options = {'shared': False}

                def generate(self):
                    tc = CMakeToolchain(self)
                    tc.generate()

                def build(self):
                    cmake = CMake(self)
                    cmake.configure()
                    cmake.build()

                def package(self):
                    cmake = CMake(self)
                    cmake.install()
            """),
        'profile_host': textwrap.dedent("""
            [settings]
            os=Android
            os.api_level=23
            arch=x86_64
            compiler=clang
            compiler.version=9
            compiler.libcxx=c++_shared
            build_type=Release
            [conf]
            tools.android:ndk_path={ndk_path}
        """.format(ndk_path=os.getenv("TEST_CONAN_ANDROID_NDK")))
    })
    return t


@pytest.mark.tool_cmake
@pytest.mark.tool_android_ndk
@pytest.mark.skipif(platform.system() != "Darwin", reason="NDK only installed on MAC")
def test_use_cmake_toolchain(client):
    """ This is the naive approach, we follow instruction from CMake in its documentation
        https://cmake.org/cmake/help/latest/manual/cmake-toolchains.7.html#cross-compiling-for-android
    """
    # Build in the cache
    client.run('create . library/version@ --profile:host=profile_host --profile:build=default')

    # Build locally
    client.run('install . library/version@ --profile:host=profile_host --profile:build=default')
    client.run_command('cmake . -DCMAKE_TOOLCHAIN_FILE={}'.format(CMakeToolchain.filename))
    client.run_command('cmake --build .')



import os
import platform
import textwrap
import time
import unittest

import pytest
from parameterized.parameterized import parameterized

from conans.model.ref import ConanFileReference, PackageReference
from conans.test.assets.cmake import gen_cmakelists
from conans.test.assets.sources import gen_function_cpp, gen_function_h
from conans.test.functional.utils import check_vs_runtime, check_exe_run
from conans.test.utils.tools import TestClient
from conans.util.files import save


@pytest.mark.tool_mingw64
@pytest.mark.tool_cmake(version="3.15")
@pytest.mark.skipif(platform.system() != "Windows", reason="Needs windows")
def test_simple_cmake_mingw():
    client = TestClient()
    client.run("new hello/1.0 -m cmake_lib")
    client.save({"mingw": """
        [settings]
        os=Windows
        arch=x86_64
        build_type=Release
        compiler=gcc
        compiler.exception=seh
        compiler.libcxx=libstdc++11
        compiler.threads=win32
        compiler.version=11.2
        cppstd=17
        """})
    client.run("create . --profile=mingw")
    # FIXME: Note that CI contains 10.X, so it uses another version rather than the profile one
    #  and no one notices. It would be good to have some details in confuser.py to be consistent
    assert "hello/1.0: __GNUC__" in client.out
    assert "hello/1.0: __MINGW" in client.out


@pytest.mark.tool_cmake
class Base(unittest.TestCase):

    conanfile = textwrap.dedent(r"""
        from conans import ConanFile
        from conan.tools.cmake import CMake, CMakeToolchain
        class App(ConanFile):
            settings = "os", "arch", "compiler", "build_type"
            requires = "hello/0.1"
            generators = "CMakeDeps"
            options = {"shared": [True, False], "fPIC": [True, False]}
            default_options = {"shared": False, "fPIC": True}

            def generate(self):
                tc = CMakeToolchain(self)
                tc.variables["MYVAR"] = "MYVAR_VALUE"
                tc.variables["MYVAR2"] = "MYVAR_VALUE2"
                tc.variables.debug["MYVAR_CONFIG"] = "MYVAR_DEBUG"
                tc.variables.release["MYVAR_CONFIG"] = "MYVAR_RELEASE"
                tc.variables.debug["MYVAR2_CONFIG"] = "MYVAR2_DEBUG"
                tc.variables.release["MYVAR2_CONFIG"] = "MYVAR2_RELEASE"
                tc.preprocessor_definitions["MYDEFINE"] = "\"MYDEF_VALUE\""
                tc.preprocessor_definitions["MYDEFINEINT"] = 42
                tc.preprocessor_definitions.debug["MYDEFINE_CONFIG"] = "\"MYDEF_DEBUG\""
                tc.preprocessor_definitions.release["MYDEFINE_CONFIG"] = "\"MYDEF_RELEASE\""
                tc.preprocessor_definitions.debug["MYDEFINEINT_CONFIG"] = 421
                tc.preprocessor_definitions.release["MYDEFINEINT_CONFIG"] = 422
                tc.generate()

            def build(self):
                cmake = CMake(self)
                cmake.configure()
                cmake.build()
        """)

    lib_h = gen_function_h(name="app")
    lib_cpp = gen_function_cpp(name="app", msg="App", includes=["hello"], calls=["hello"],
                               preprocessor=["MYVAR", "MYVAR_CONFIG", "MYDEFINE", "MYDEFINE_CONFIG",
                                             "MYDEFINEINT", "MYDEFINEINT_CONFIG"])
    main = gen_function_cpp(name="main", includes=["app"], calls=["app"])

    cmakelist = textwrap.dedent("""
        cmake_minimum_required(VERSION 3.15)
        project(App C CXX)

        if(NOT CMAKE_TOOLCHAIN_FILE)
            message(FATAL ">> Not using toolchain")
        endif()
        message(">> CMAKE_GENERATOR_PLATFORM: ${CMAKE_GENERATOR_PLATFORM}")
        message(">> CMAKE_BUILD_TYPE: ${CMAKE_BUILD_TYPE}")
        message(">> CMAKE_CXX_FLAGS: ${CMAKE_CXX_FLAGS}")
        message(">> CMAKE_CXX_FLAGS_DEBUG: ${CMAKE_CXX_FLAGS_DEBUG}")
        message(">> CMAKE_CXX_FLAGS_RELEASE: ${CMAKE_CXX_FLAGS_RELEASE}")
        message(">> CMAKE_C_FLAGS: ${CMAKE_C_FLAGS}")
        message(">> CMAKE_C_FLAGS_DEBUG: ${CMAKE_C_FLAGS_DEBUG}")
        message(">> CMAKE_C_FLAGS_RELEASE: ${CMAKE_C_FLAGS_RELEASE}")
        message(">> CMAKE_SHARED_LINKER_FLAGS: ${CMAKE_SHARED_LINKER_FLAGS}")
        message(">> CMAKE_EXE_LINKER_FLAGS: ${CMAKE_EXE_LINKER_FLAGS}")
        message(">> CMAKE_CXX_STANDARD: ${CMAKE_CXX_STANDARD}")
        message(">> CMAKE_CXX_EXTENSIONS: ${CMAKE_CXX_EXTENSIONS}")
        message(">> CMAKE_POSITION_INDEPENDENT_CODE: ${CMAKE_POSITION_INDEPENDENT_CODE}")
        message(">> CMAKE_SKIP_RPATH: ${CMAKE_SKIP_RPATH}")
        message(">> CMAKE_INSTALL_NAME_DIR: ${CMAKE_INSTALL_NAME_DIR}")
        message(">> CMAKE_MODULE_PATH: ${CMAKE_MODULE_PATH}")
        message(">> CMAKE_PREFIX_PATH: ${CMAKE_PREFIX_PATH}")
        message(">> BUILD_SHARED_LIBS: ${BUILD_SHARED_LIBS}")
        get_directory_property(_COMPILE_DEFS DIRECTORY ${CMAKE_SOURCE_DIR} COMPILE_DEFINITIONS)
        message(">> COMPILE_DEFINITIONS: ${_COMPILE_DEFS}")

        find_package(hello REQUIRED)
        add_library(app_lib app_lib.cpp)
        target_link_libraries(app_lib PRIVATE hello::hello)
        target_compile_definitions(app_lib PRIVATE MYVAR="${MYVAR}")
        target_compile_definitions(app_lib PRIVATE MYVAR_CONFIG="${MYVAR_CONFIG}")
        add_executable(app app.cpp)
        target_link_libraries(app PRIVATE app_lib)
        """)

    def setUp(self):
        self.client = TestClient()
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            from conans.tools import save
            import os
            class Pkg(ConanFile):
                settings = "build_type"
                def package(self):
                    save(os.path.join(self.package_folder, "include/hello.h"),
                         '''#include <iostream>
                         void hello(){std::cout<< "Hello: %s" <<std::endl;}'''
                         % self.settings.build_type)
            """)
        self.client.save({"conanfile.py": conanfile})
        self.client.run("create . hello/0.1@ -s build_type=Debug")
        self.client.run("create . hello/0.1@ -s build_type=Release")

        # Prepare the actual consumer package
        self.client.save({"conanfile.py": self.conanfile,
                          "CMakeLists.txt": self.cmakelist,
                          "app.cpp": self.main,
                          "app_lib.cpp": self.lib_cpp,
                          "app.h": self.lib_h})

    def _run_build(self, settings=None, options=None):
        # Build the profile according to the settings provided
        settings = settings or {}
        settings = " ".join('-s %s="%s"' % (k, v) for k, v in settings.items() if v)
        options = " ".join("-o %s=%s" % (k, v) for k, v in options.items()) if options else ""

        # Run the configure corresponding to this test case
        build_directory = os.path.join(self.client.current_folder, "build").replace("\\", "/")
        with self.client.chdir(build_directory):
            self.client.run("install .. %s %s" % (settings, options))
            install_out = self.client.out
            self.client.run("build ..")
        return install_out

    def _modify_code(self):
        lib_cpp = gen_function_cpp(name="app", msg="AppImproved", includes=["hello"],
                                   calls=["hello"], preprocessor=["MYVAR", "MYVAR_CONFIG",
                                                                  "MYDEFINE", "MYDEFINE_CONFIG",
                                                                  "MYDEFINEINT",
                                                                  "MYDEFINEINT_CONFIG"])
        self.client.save({"app_lib.cpp": lib_cpp})

        content = self.client.load("CMakeLists.txt")
        content = content.replace(">>", "++>>")
        self.client.save({"CMakeLists.txt": content})

    def _incremental_build(self, build_type=None):
        build_directory = os.path.join(self.client.current_folder, "build").replace("\\", "/")
        with self.client.chdir(build_directory):
            config = "--config %s" % build_type if build_type else ""
            self.client.run_command("cmake --build . %s" % config)

    def _run_app(self, build_type, bin_folder=False, msg="App", dyld_path=None):
        if dyld_path:
            build_directory = os.path.join(self.client.current_folder, "build").replace("\\", "/")
            command_str = 'DYLD_LIBRARY_PATH="%s" build/app' % build_directory
        else:
            command_str = "build/%s/app.exe" % build_type if bin_folder else "build/app"
            if platform.system() == "Windows":
                command_str = command_str.replace("/", "\\")
        self.client.run_command(command_str)
        self.assertIn("Hello: %s" % build_type, self.client.out)
        self.assertIn("%s: %s!" % (msg, build_type), self.client.out)
        self.assertIn("MYVAR: MYVAR_VALUE", self.client.out)
        self.assertIn("MYVAR_CONFIG: MYVAR_%s" % build_type.upper(), self.client.out)
        self.assertIn("MYDEFINE: MYDEF_VALUE", self.client.out)
        self.assertIn("MYDEFINE_CONFIG: MYDEF_%s" % build_type.upper(), self.client.out)
        self.assertIn("MYDEFINEINT: 42", self.client.out)
        self.assertIn("MYDEFINEINT_CONFIG: {}".format(421 if build_type == "Debug" else 422),
                      self.client.out)


@pytest.mark.skipif(platform.system() != "Windows", reason="Only for windows")
class WinTest(Base):
    @parameterized.expand([("Visual Studio", "Debug", "MTd", "15", "14", "x86", "v140", True),
                           ("Visual Studio", "Release", "MD", "15", "17", "x86_64", "", False),
                           ("msvc", "Debug", "static", "191", "14", "x86", None, True),
                           ("msvc", "Release", "dynamic", "191", "17", "x86_64", None, False)]
                          )
    def test_toolchain_win(self, compiler, build_type, runtime, version, cppstd, arch, toolset,
                           shared):
        settings = {"compiler": compiler,
                    "compiler.version": version,
                    "compiler.toolset": toolset,
                    "compiler.runtime": runtime,
                    "compiler.cppstd": cppstd,
                    "arch": arch,
                    "build_type": build_type,
                    }
        options = {"shared": shared}
        save(self.client.cache.new_config_path, "tools.build:jobs=1")
        install_out = self._run_build(settings, options)
        self.assertIn("WARN: Toolchain: Ignoring fPIC option defined for Windows", install_out)

        # FIXME: Hardcoded VS version and partial toolset check
        toolchain_path = os.path.join(self.client.current_folder, "build",
                                      "conan_toolchain.cmake").replace("\\", "/")
        self.assertIn('CMake command: cmake -G "Visual Studio 15 2017" '
                      '-DCMAKE_TOOLCHAIN_FILE="{}"'.format(toolchain_path), self.client.out)
        if toolset == "v140":
            self.assertIn("Microsoft Visual Studio 14.0", self.client.out)
        else:
            self.assertIn("Microsoft Visual Studio/2017", self.client.out)

        generator_platform = "x64" if arch == "x86_64" else "Win32"
        arch_flag = "x64" if arch == "x86_64" else "X86"
        shared_str = "ON" if shared else "OFF"
        vals = {"CMAKE_GENERATOR_PLATFORM": generator_platform,
                "CMAKE_BUILD_TYPE": "",
                "CMAKE_CXX_FLAGS": "/MP1 /DWIN32 /D_WINDOWS /GR /EHsc",
                "CMAKE_CXX_FLAGS_DEBUG": "/Zi /Ob0 /Od /RTC1",
                "CMAKE_CXX_FLAGS_RELEASE": "/O2 /Ob2 /DNDEBUG",
                "CMAKE_C_FLAGS": "/MP1 /DWIN32 /D_WINDOWS",
                "CMAKE_C_FLAGS_DEBUG": "/Zi /Ob0 /Od /RTC1",
                "CMAKE_C_FLAGS_RELEASE": "/O2 /Ob2 /DNDEBUG",
                "CMAKE_SHARED_LINKER_FLAGS": "/machine:%s" % arch_flag,
                "CMAKE_EXE_LINKER_FLAGS": "/machine:%s" % arch_flag,
                "CMAKE_CXX_STANDARD": cppstd,
                "CMAKE_CXX_EXTENSIONS": "OFF",
                "BUILD_SHARED_LIBS": shared_str}

        def _verify_out(marker=">>"):
            if shared:
                self.assertIn("app_lib.dll", self.client.out)
            else:
                self.assertNotIn("app_lib.dll", self.client.out)

            out = str(self.client.out).splitlines()
            for k, v in vals.items():
                self.assertIn("%s %s: %s" % (marker, k, v), out)

        _verify_out()

        opposite_build_type = "Release" if build_type == "Debug" else "Debug"
        settings["build_type"] = opposite_build_type
        if runtime == "MTd":
            settings["compiler.runtime"] = "MT"
        if runtime == "MD":
            settings["compiler.runtime"] = "MDd"
        self._run_build(settings, options)

        self._run_app("Release", bin_folder=True)
        if compiler == "msvc":
            visual_version = version
        else:
            visual_version = "190" if toolset == "v140" else "191"
        check_exe_run(self.client.out, "main", "msvc", visual_version, "Release", arch, cppstd,
                      {"MYVAR": "MYVAR_VALUE",
                       "MYVAR_CONFIG": "MYVAR_RELEASE",
                       "MYDEFINE": "MYDEF_VALUE",
                       "MYDEFINE_CONFIG": "MYDEF_RELEASE"
                       })
        self._run_app("Debug", bin_folder=True)
        check_exe_run(self.client.out, "main", "msvc", visual_version, "Debug", arch, cppstd,
                      {"MYVAR": "MYVAR_VALUE",
                       "MYVAR_CONFIG": "MYVAR_DEBUG",
                       "MYDEFINE": "MYDEF_VALUE",
                       "MYDEFINE_CONFIG": "MYDEF_DEBUG"
                       })

        static_runtime = True if runtime == "static" or "MT" in runtime else False
        check_vs_runtime("build/Release/app.exe", self.client, "15", build_type="Release",
                         static_runtime=static_runtime)
        check_vs_runtime("build/Debug/app.exe", self.client, "15", build_type="Debug",
                         static_runtime=static_runtime)

        self._modify_code()
        time.sleep(1)
        self._incremental_build(build_type=build_type)
        _verify_out(marker="++>>")
        self._run_app(build_type, bin_folder=True, msg="AppImproved")
        self._incremental_build(build_type=opposite_build_type)
        self._run_app(opposite_build_type, bin_folder=True, msg="AppImproved")

    @parameterized.expand([("Debug", "libstdc++", "4.9", "98", "x86_64", True),
                           ("Release", "libstdc++", "4.9", "11", "x86_64", False)])
    @pytest.mark.tool_mingw64
    @pytest.mark.tool_cmake(version="3.15")
    def test_toolchain_mingw_win(self, build_type, libcxx, version, cppstd, arch, shared):
        # FIXME: The version and cppstd are wrong, toolchain doesn't enforce it
        settings = {"compiler": "gcc",
                    "compiler.version": version,
                    "compiler.libcxx": libcxx,
                    "compiler.cppstd": cppstd,
                    "arch": arch,
                    "build_type": build_type,
                    }
        options = {"shared": shared}
        install_out = self._run_build(settings, options)
        self.assertIn("WARN: Toolchain: Ignoring fPIC option defined for Windows", install_out)
        self.assertIn("The C compiler identification is GNU", self.client.out)
        toolchain_path = os.path.join(self.client.current_folder, "build",
                                      "conan_toolchain.cmake").replace("\\", "/")
        self.assertIn('CMake command: cmake -G "MinGW Makefiles" '
                      '-DCMAKE_TOOLCHAIN_FILE="{}"'.format(toolchain_path), self.client.out)
        assert '-DCMAKE_SH="CMAKE_SH-NOTFOUND"' in self.client.out

        def _verify_out(marker=">>"):
            cmake_vars = {"CMAKE_GENERATOR_PLATFORM": "",
                          "CMAKE_BUILD_TYPE": build_type,
                          "CMAKE_CXX_FLAGS": "-m64",
                          "CMAKE_CXX_FLAGS_DEBUG": "-g",
                          "CMAKE_CXX_FLAGS_RELEASE": "-O3 -DNDEBUG",
                          "CMAKE_C_FLAGS": "-m64",
                          "CMAKE_C_FLAGS_DEBUG": "-g",
                          "CMAKE_C_FLAGS_RELEASE": "-O3 -DNDEBUG",
                          "CMAKE_SHARED_LINKER_FLAGS": "-m64",
                          "CMAKE_EXE_LINKER_FLAGS": "-m64",
                          "CMAKE_CXX_STANDARD": cppstd,
                          "CMAKE_CXX_EXTENSIONS": "OFF",
                          "BUILD_SHARED_LIBS": "ON" if shared else "OFF"}
            if shared:
                self.assertIn("app_lib.dll", self.client.out)
            else:
                self.assertNotIn("app_lib.dll", self.client.out)

            out = str(self.client.out).splitlines()
            for k, v in cmake_vars.items():
                self.assertIn("%s %s: %s" % (marker, k, v), out)

        _verify_out()
        self._run_app(build_type)
        check_exe_run(self.client.out, "main", "gcc", None, build_type, arch, None,
                      {"MYVAR": "MYVAR_VALUE",
                       "MYVAR_CONFIG": "MYVAR_{}".format(build_type.upper()),
                       "MYDEFINE": "MYDEF_VALUE",
                       "MYDEFINE_CONFIG": "MYDEF_{}".format(build_type.upper())
                       }, subsystem="mingw64")

        self._modify_code()
        time.sleep(2)
        self._incremental_build()
        _verify_out(marker="++>>")
        self._run_app(build_type, msg="AppImproved")


@pytest.mark.skipif(platform.system() != "Linux", reason="Only for Linux")
class LinuxTest(Base):
    @parameterized.expand([("Debug",  "14", "x86", "libstdc++", True),
                           ("Release", "gnu14", "x86_64", "libstdc++11", False)])
    def test_toolchain_linux(self, build_type, cppstd, arch, libcxx, shared):
        settings = {"compiler": "gcc",
                    "compiler.cppstd": cppstd,
                    "compiler.libcxx": libcxx,
                    "arch": arch,
                    "build_type": build_type}
        self._run_build(settings, {"shared": shared})
        toolchain_path = os.path.join(self.client.current_folder, "build",
                                      "conan_toolchain.cmake").replace("\\", "/")
        self.assertIn('CMake command: cmake -G "Unix Makefiles" '
                      '-DCMAKE_TOOLCHAIN_FILE="{}"'.format(toolchain_path), self.client.out)

        extensions_str = "ON" if "gnu" in cppstd else "OFF"
        arch_str = "-m32" if arch == "x86" else "-m64"
        cxx11_abi_str = "_GLIBCXX_USE_CXX11_ABI=0;" if libcxx == "libstdc++" else ""
        defines = '%sMYDEFINE="MYDEF_VALUE";MYDEFINEINT=42;'\
                  'MYDEFINE_CONFIG=$<IF:$<CONFIG:debug>,"MYDEF_DEBUG",$<IF:$<CONFIG:release>,'\
                  '"MYDEF_RELEASE","">>;MYDEFINEINT_CONFIG=$<IF:$<CONFIG:debug>,421,'\
                  '$<IF:$<CONFIG:release>,422,"">>' % cxx11_abi_str
        vals = {"CMAKE_CXX_STANDARD": "14",
                "CMAKE_CXX_EXTENSIONS": extensions_str,
                "CMAKE_BUILD_TYPE": build_type,
                "CMAKE_CXX_FLAGS": arch_str,
                "CMAKE_CXX_FLAGS_DEBUG": "-g",
                "CMAKE_CXX_FLAGS_RELEASE": "-O3 -DNDEBUG",
                "CMAKE_C_FLAGS": arch_str,
                "CMAKE_C_FLAGS_DEBUG": "-g",
                "CMAKE_C_FLAGS_RELEASE": "-O3 -DNDEBUG",
                "CMAKE_SHARED_LINKER_FLAGS": arch_str,
                "CMAKE_EXE_LINKER_FLAGS": arch_str,
                "COMPILE_DEFINITIONS": defines,
                "CMAKE_POSITION_INDEPENDENT_CODE": "ON"
                }

        def _verify_out(marker=">>"):
            if shared:
                self.assertIn("libapp_lib.so", self.client.out)
            else:
                self.assertIn("libapp_lib.a", self.client.out)

            out = str(self.client.out).splitlines()
            for k, v in vals.items():
                self.assertIn("%s %s: %s" % (marker, k, v), out)

        _verify_out()

        self._run_app(build_type)

        self._modify_code()
        self._incremental_build()
        _verify_out(marker="++>>")
        self._run_app(build_type, msg="AppImproved")


@pytest.mark.skipif(platform.system() != "Darwin", reason="Only for Apple")
class AppleTest(Base):
    @parameterized.expand([("Debug",  "14",  True),
                           ("Release", "", False)])
    def test_toolchain_apple(self, build_type, cppstd, shared):
        settings = {"compiler": "apple-clang",
                    "compiler.cppstd": cppstd,
                    "build_type": build_type}
        self._run_build(settings, {"shared": shared})

        toolchain_path = os.path.join(self.client.current_folder, "build",
                                      "conan_toolchain.cmake").replace("\\", "/")
        self.assertIn('CMake command: cmake -G "Unix Makefiles" '
                      '-DCMAKE_TOOLCHAIN_FILE="{}"'.format(toolchain_path), self.client.out)

        extensions_str = "OFF" if cppstd else ""
        vals = {"CMAKE_CXX_STANDARD": cppstd,
                "CMAKE_CXX_EXTENSIONS": extensions_str,
                "CMAKE_BUILD_TYPE": build_type,
                "CMAKE_CXX_FLAGS_DEBUG": "-g",
                "CMAKE_CXX_FLAGS_RELEASE": "-O3 -DNDEBUG",
                "CMAKE_C_FLAGS_DEBUG": "-g",
                "CMAKE_C_FLAGS_RELEASE": "-O3 -DNDEBUG",
                "CMAKE_INSTALL_NAME_DIR": ""
                }

        host_arch = self.client.get_default_host_profile().settings['arch']

        if host_arch == "x86_64":
            vals.update({
                "CMAKE_C_FLAGS": "-m64",
                "CMAKE_CXX_FLAGS": "-m64 -stdlib=libc++",
                "CMAKE_SHARED_LINKER_FLAGS": "-m64",
                "CMAKE_EXE_LINKER_FLAGS": "-m64",
            })
        else:
            vals.update({
                "CMAKE_CXX_FLAGS": "-stdlib=libc++",
            })

        def _verify_out(marker=">>"):
            if shared:
                self.assertIn("libapp_lib.dylib", self.client.out)
            else:
                if marker == ">>":
                    self.assertIn("libapp_lib.a", self.client.out)
                else:  # Incremental build not the same msg
                    self.assertIn("Built target app_lib", self.client.out)
            out = str(self.client.out).splitlines()
            for k, v in vals.items():
                self.assertIn("%s %s: %s" % (marker, k, v), out)

        _verify_out()

        self._run_app(build_type, dyld_path=shared)

        self._modify_code()
        time.sleep(1)
        self._incremental_build()
        _verify_out(marker="++>>")
        self._run_app(build_type, dyld_path=shared, msg="AppImproved")


@pytest.mark.skipif(platform.system() != "Windows", reason="Only for windows")
@pytest.mark.parametrize("version, vs_version",
                         [("190", "15"),
                          ("191", "15")])
def test_msvc_vs_versiontoolset(version, vs_version):
    settings = {"compiler": "msvc",
                "compiler.version": version,
                "compiler.runtime": "static",
                "compiler.cppstd": "14",
                "arch": "x86_64",
                "build_type": "Release",
                }
    client = TestClient()
    save(client.cache.new_config_path,
         "tools.microsoft.msbuild:vs_version={}".format(vs_version))
    conanfile = textwrap.dedent("""
            from conans import ConanFile
            from conan.tools.cmake import CMake
            class App(ConanFile):
                settings = "os", "arch", "compiler", "build_type"
                generators = "CMakeToolchain"
                options = {"shared": [True, False], "fPIC": [True, False]}
                default_options = {"shared": False, "fPIC": True}
                exports_sources = "*"

                def build(self):
                    cmake = CMake(self)
                    cmake.configure()
                    cmake.build()
                    self.run("Release\\myapp.exe")
            """)
    cmakelists = gen_cmakelists(appname="myapp", appsources=["app.cpp"])
    main = gen_function_cpp(name="main")
    client.save({"conanfile.py": conanfile,
                 "CMakeLists.txt": cmakelists,
                 "app.cpp": main,
                 })
    settings = " ".join('-s %s="%s"' % (k, v) for k, v in settings.items() if v)
    client.run("create . app/1.0@ {}".format(settings))
    assert '-G "Visual Studio 15 2017"' in client.out

    check_exe_run(client.out, "main", "msvc", version, "Release", "x86_64", "14")


@pytest.mark.tool_cmake
class CMakeInstallTest(unittest.TestCase):

    def test_install(self):
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            from conan.tools.cmake import CMake, CMakeToolchain
            class App(ConanFile):
                settings = "os", "arch", "compiler", "build_type"
                exports_sources = "CMakeLists.txt", "header.h"
                def generate(self):
                    tc = CMakeToolchain(self)
                    tc.generate()
                def build(self):
                    cmake = CMake(self)
                    cmake.configure()
                def package(self):
                    cmake = CMake(self)
                    cmake.install()
            """)

        cmakelist = textwrap.dedent("""
            cmake_minimum_required(VERSION 2.8)
            project(App C)
            if(CONAN_TOOLCHAIN_INCLUDED AND CMAKE_VERSION VERSION_LESS "3.15")
                include("${CMAKE_BINARY_DIR}/conan_project_include.cmake")
            endif()
            if(NOT CMAKE_TOOLCHAIN_FILE)
                message(FATAL ">> Not using toolchain")
            endif()
            install(FILES header.h DESTINATION include)
            """)
        client = TestClient(path_with_spaces=False)
        client.save({"conanfile.py": conanfile,
                     "CMakeLists.txt": cmakelist,
                     "header.h": "# my header file"})

        # FIXME: This is broken, because the toolchain at install time, doesn't have the package
        # folder yet. We need to define the layout for local development
        """
        with client.chdir("build"):
            client.run("install ..")
            client.run("build ..")
            client.run("package .. -pf=mypkg")  # -pf=mypkg ignored
        self.assertTrue(os.path.exists(os.path.join(client.current_folder, "build",
                                                    "include", "header.h")))"""

        # The create flow must work
        client.run("create . pkg/0.1@")
        self.assertIn("pkg/0.1 package(): Packaged 1 '.h' file: header.h", client.out)
        ref = ConanFileReference.loads("pkg/0.1")
        layout = client.cache.package_layout(ref)
        package_id = layout.package_ids()[0]
        package_folder = layout.package(PackageReference(ref, package_id))
        self.assertTrue(os.path.exists(os.path.join(package_folder, "include", "header.h")))


@pytest.mark.tool_cmake
class CMakeOverrideCacheTest(unittest.TestCase):

    def test_cmake_cache_variables(self):
        # https://github.com/conan-io/conan/issues/7832
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            from conan.tools.cmake import CMake, CMakeToolchain
            class App(ConanFile):
                settings = "os", "arch", "compiler", "build_type"
                exports_sources = "CMakeLists.txt"
                def generate(self):
                    toolchain = CMakeToolchain(self)
                    toolchain.variables["my_config_string"] = "my new value"
                    toolchain.generate()
                def build(self):
                    cmake = CMake(self)
                    cmake.configure()
            """)

        cmakelist = textwrap.dedent("""
            cmake_minimum_required(VERSION 3.7)
            project(my_project)
            set(my_config_string "default value" CACHE STRING "my config string")
            message(STATUS "VALUE OF CONFIG STRING: ${my_config_string}")
            """)
        client = TestClient()
        client.save({"conanfile.py": conanfile,
                     "CMakeLists.txt": cmakelist})
        client.run("install .")
        client.run("build .")
        self.assertIn("VALUE OF CONFIG STRING: my new value", client.out)


@pytest.mark.tool_cmake
class TestCMakeFindPackagePreferConfig:

    def test_prefer_config(self):
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            from conan.tools.cmake import CMake
            class App(ConanFile):
                settings = "os", "arch", "compiler", "build_type"
                generators = "CMakeToolchain"
                def build(self):
                    cmake = CMake(self)
                    cmake.configure()
            """)

        cmakelist = textwrap.dedent("""
            set(CMAKE_C_COMPILER_WORKS 1)
            set(CMAKE_C_ABI_COMPILED 1)
            cmake_minimum_required(VERSION 3.15)
            project(my_project C)
            find_package(Comandante REQUIRED)
            """)

        find = 'message(STATUS "using FindComandante.cmake")'
        config = 'message(STATUS "using ComandanteConfig.cmake")'

        profile = textwrap.dedent("""
            include(default)
            [conf]
            tools.cmake.cmaketoolchain:find_package_prefer_config={}
            """)

        client = TestClient()
        client.save({"conanfile.py": conanfile,
                     "CMakeLists.txt": cmakelist,
                     "FindComandante.cmake": find,
                     "ComandanteConfig.cmake": config,
                     "profile_true": profile.format(True),
                     "profile_false": profile.format(False)})

        client.run("install .")
        client.run("build .")
        assert "using ComandanteConfig.cmake" in client.out

        client.run("install . --profile=profile_true")
        client.run("build .")
        assert "using ComandanteConfig.cmake" in client.out

        client.run("install . --profile=profile_false")
        client.run("build .")
        assert "using FindComandante.cmake" in client.out



# coding=utf-8

import os
import unittest

import mock
import pytest

from conans.client.source import _run_cache_scm, _run_local_scm
from conans.client.tools.scm import Git
from conans.model.scm import SCM
from conans.test.utils.test_files import temp_folder
from conans.test.utils.mocks import TestBufferConanOutput
from conans.test.utils.scm import create_local_git_repo


@pytest.mark.tool_git
class RunSCMTest(unittest.TestCase):

    def setUp(self):
        self.tmp_dir = temp_folder()
        self.src_folder = os.path.join(self.tmp_dir, 'source')

    def test_in_cache_with_local_sources(self):
        """In cache, if we have cached scm sources in the scm_sources, it will get them"""
        output = TestBufferConanOutput()
        local_sources_path = self.tmp_dir.replace('\\', '/')

        # Mock the conanfile (return scm_data)
        conanfile = mock.MagicMock()
        conanfile.scm = {'type': 'git', 'url': 'auto', 'revision': 'auto'}
        conanfile.folders.base_source = self.src_folder

        # Mock functions called from inside _run_scm (tests will be here)
        def merge_directories(src, dst, excluded=None):
            self.assertEqual(src, local_sources_path)
            self.assertEqual(dst, self.src_folder)

        def clean_source_folder(folder):
            self.assertEqual(folder, self.src_folder)

        with mock.patch("conans.client.source.merge_directories", side_effect=merge_directories):
            with mock.patch("conans.client.source._clean_source_folder",
                            side_effect=clean_source_folder):

                _run_cache_scm(conanfile,
                               scm_sources_folder=local_sources_path,
                               output=output)

        self.assertIn("Copying previously cached scm sources", output)

    def test_in_cache_no_local_sources(self):
        """In cache, if we DON'T have cached scm sources in the scm_sources, it will clone"""
        output = TestBufferConanOutput()

        # Mock the conanfile (return scm_data)
        subfolder = 'subfolder'
        url = 'whatever'
        conanfile = mock.MagicMock()
        conanfile.scm = {'type': 'git', 'url': url, 'revision': 'auto', 'subfolder': subfolder}
        conanfile.folders.base_source = self.src_folder

        # Mock functions called from inside _run_scm (tests will be here)
        def clean_source_folder(folder):
            self.assertEqual(folder, os.path.join(self.src_folder, subfolder))

        def scm_checkout(scm_itself):
            self.assertEqual(scm_itself.repo_folder, os.path.join(self.src_folder, subfolder))

        with mock.patch("conans.client.source._clean_source_folder",
                        side_effect=clean_source_folder):
            with mock.patch.object(SCM, "checkout", new=scm_checkout):
                _run_cache_scm(conanfile,
                               scm_sources_folder="/not/existing/path",
                               output=output)

        self.assertIn("SCM: Getting sources from url: '{}'".format(url), output)

    def test_user_space_with_local_sources(self):
        output = TestBufferConanOutput()

        # Need a real repo to get a working SCM object
        local_sources_path = os.path.join(self.tmp_dir, 'git_repo').replace('\\', '/')
        create_local_git_repo(files={'file1': "content"}, folder=local_sources_path)
        git = Git(local_sources_path)
        url = "https://remote.url"
        git.run("remote add origin \"{}\"".format(url))

        # Mock the conanfile (return scm_data)
        conanfile = mock.MagicMock()
        conanfile.scm = {'type': 'git', 'url': 'auto', 'revision': 'auto'}

        # Mock functions called from inside _run_scm (tests will be here)
        def merge_directories(src, dst, excluded=None):
            src = os.path.normpath(src)
            dst = os.path.normpath(dst)
            self.assertEqual(src.replace('\\', '/'), local_sources_path)
            self.assertEqual(dst, self.src_folder)

        with mock.patch("conans.client.source.merge_directories", side_effect=merge_directories):
            _run_local_scm(conanfile,
                           conanfile_folder=local_sources_path,
                           src_folder=self.src_folder,
                           output=output)

        self.assertIn("getting sources from folder: {}".format(local_sources_path).lower(),
                      str(output).lower())



import six

from conans import tools
from conans.errors import ConanV2Exception
from conans.test.utils.conan_v2_tests import ConanV2ModeTestCase


class ToolsWinTestCase(ConanV2ModeTestCase):
    def test_msvc_build_command(self):
        with six.assertRaisesRegex(self, ConanV2Exception, "Conan v2 incompatible: 'tools.msvc_build_command' is deprecated"):
            tools.msvc_build_command(settings=None, sln_path=None)

    def test_build_sln_command(self):
        with six.assertRaisesRegex(self, ConanV2Exception, "Conan v2 incompatible: 'tools.build_sln_command' is deprecated"):
            tools.build_sln_command(settings=None, sln_path=None)



import unittest
from textwrap import dedent

import pytest

from conans.model.ref import ConanFileReference
from conans.test.utils.tools import TestClient, GenConanfile
from conans.util.env_reader import get_env


class FullRevisionModeTest(unittest.TestCase):

    def test_recipe_revision_mode(self):
        liba_ref = ConanFileReference.loads("liba/0.1@user/testing")
        libb_ref = ConanFileReference.loads("libb/0.1@user/testing")

        clienta = TestClient()
        clienta.run("config set general.default_package_id_mode=recipe_revision_mode")
        conanfilea = dedent("""
            from conans import ConanFile
            from conans.tools import save
            import uuid, os
            class Pkg(ConanFile):
                def package(self):
                    save(os.path.join(self.package_folder, "file.txt"),
                         str(uuid.uuid1()))
            """)
        clienta.save({"conanfile.py": conanfilea})
        clienta.run("create . liba/0.1@user/testing")

        clientb = TestClient(cache_folder=clienta.cache_folder)
        clientb.save({"conanfile.py": GenConanfile().with_name("libb").with_version("0.1")
                                                    .with_require(liba_ref)})
        clientb.run("create . user/testing")

        clientc = TestClient(cache_folder=clienta.cache_folder)
        clientc.save({"conanfile.py": GenConanfile().with_name("libc").with_version("0.1")
                                                    .with_require(libb_ref)})
        clientc.run("install . user/testing")

        # Do a minor change to the recipe, it will change the recipe revision
        clienta.save({"conanfile.py": conanfilea + "# comment"})
        clienta.run("create . liba/0.1@user/testing")

        clientc.run("install . user/testing", assert_error=True)
        self.assertIn("ERROR: Missing prebuilt package for 'libb/0.1@user/testing'", clientc.out)
        # Building b with the new recipe revision of liba works
        clientc.run("install . user/testing --build=libb")

        # Now change only the package revision of liba
        clienta.run("create . liba/0.1@user/testing")
        clientc.run("install . user/testing")
        clientc.run("config set general.default_package_id_mode=package_revision_mode")
        clientc.run("install . user/testing", assert_error=True)
        self.assertIn("ERROR: Missing prebuilt package for 'libb/0.1@user/testing'", clientc.out)
        clientc.run("install . user/testing --build=libb")
        clientc.run("info . --build-order=ALL")

        clienta.run("create . liba/0.1@user/testing")
        clientc.run("install . user/testing", assert_error=True)
        self.assertIn("ERROR: Missing prebuilt package for 'libb/0.1@user/testing'", clientc.out)

        clienta.run("create . liba/0.1@user/testing")
        clientc.run("info . --build-order=ALL")

    def test_binary_id_recomputation_after_build(self):
        clienta = TestClient()
        clienta.run("config set general.default_package_id_mode=recipe_revision_mode")
        conanfile = dedent("""
            from conans import ConanFile
            from conans.tools import save
            import uuid, os
            class Pkg(ConanFile):
                %s
                def package(self):
                    save(os.path.join(self.package_folder, "file.txt"),
                         str(uuid.uuid1()))
            """)
        clienta.save({"conanfile.py": conanfile % ""})
        clienta.run("create . liba/0.1@user/testing")

        clientb = TestClient(cache_folder=clienta.cache_folder)
        clientb.save({"conanfile.py": conanfile % "requires = 'liba/0.1@user/testing'"})
        clientb.run("config set general.default_package_id_mode=package_revision_mode")
        clientb.run("create . libb/0.1@user/testing")

        clientc = TestClient(cache_folder=clienta.cache_folder)
        clientc.save({"conanfile.py": conanfile % "requires = 'libb/0.1@user/testing'"})
        clientc.run("config set general.default_package_id_mode=package_revision_mode")
        clientc.run("create . libc/0.1@user/testing")

        clientd = TestClient(cache_folder=clienta.cache_folder)
        clientd.run("config set general.default_package_id_mode=package_revision_mode")
        clientd.save({"conanfile.py": conanfile % "requires = 'libc/0.1@user/testing'"})
        clientd.run("install . libd/0.1@user/testing")

        # Change A PREV
        clienta.run("create . liba/0.1@user/testing")
        clientd.run("install . libd/0.1@user/testing", assert_error=True)
        self.assertIn("ERROR: Missing prebuilt package for 'libb/0.1@user/testing'", clientd.out)
        clientd.run("install . libd/0.1@user/testing --build=missing")

        self.assertIn("libc/0.1@user/testing: Unknown binary", clientd.out)
        self.assertIn("libc/0.1@user/testing: Updated ID", clientd.out)
        self.assertIn("libc/0.1@user/testing: Binary for the updated ID has to be built",
                      clientd.out)
        self.assertIn("libc/0.1@user/testing: Calling build()", clientd.out)

    def test_binary_id_recomputation_with_build_requires(self):
        clienta = TestClient()
        clienta.save({"conanfile.py": GenConanfile().with_name("Tool").with_version("0.1")
                                                    .with_package_info(cpp_info={"libs":
                                                                                 ["tool.lib"]},
                                                                       env_info={})})
        clienta.run("create . user/testing")
        clienta.run("config set general.default_package_id_mode=recipe_revision_mode")
        conanfile = dedent("""
            from conans import ConanFile
            from conans.tools import save
            import uuid, os
            class Pkg(ConanFile):
                build_requires = "Tool/0.1@user/testing"
                %s
                def build(self):
                    self.output.info("TOOLS LIBS: {}".format(self.deps_cpp_info["Tool"].libs))
                def package(self):
                    save(os.path.join(self.package_folder, "file.txt"),
                         str(uuid.uuid1()))
            """)
        clienta.save({"conanfile.py": conanfile % ""})
        clienta.run("create . liba/0.1@user/testing")

        clientb = TestClient(cache_folder=clienta.cache_folder)
        clientb.save({"conanfile.py": conanfile % "requires = 'liba/0.1@user/testing'"})
        clientb.run("config set general.default_package_id_mode=package_revision_mode")
        clientb.run("create . libb/0.1@user/testing")

        clientc = TestClient(cache_folder=clienta.cache_folder)
        clientc.save({"conanfile.py": conanfile % "requires = 'libb/0.1@user/testing'"})
        clientc.run("config set general.default_package_id_mode=package_revision_mode")
        clientc.run("create . libc/0.1@user/testing")

        clientd = TestClient(cache_folder=clienta.cache_folder)
        clientd.run("config set general.default_package_id_mode=package_revision_mode")
        clientd.save({"conanfile.py": conanfile % "requires = 'libc/0.1@user/testing'"})
        clientd.run("install . libd/0.1@user/testing")

        # Change A PREV
        clienta.run("create . liba/0.1@user/testing")
        clientd.run("install . libd/0.1@user/testing", assert_error=True)
        self.assertIn("ERROR: Missing prebuilt package for 'libb/0.1@user/testing'", clientd.out)
        clientd.run("install . libd/0.1@user/testing --build=missing")

        self.assertIn("libc/0.1@user/testing: Unknown binary", clientd.out)
        self.assertIn("libc/0.1@user/testing: Updated ID", clientd.out)
        self.assertIn("libc/0.1@user/testing: Binary for the updated ID has to be built",
                      clientd.out)
        self.assertIn("libc/0.1@user/testing: Calling build()", clientd.out)

    def test_reusing_artifacts_after_build(self):
        # An unknown binary that after build results in the exact same PREF with PREV, doesn't
        # fire build of downstream
        client = TestClient()
        client.run("config set general.default_package_id_mode=package_revision_mode")
        client.save({"conanfile.py": GenConanfile()})
        client.run("create . liba/0.1@user/testing")

        client.save({"conanfile.py": GenConanfile().with_require('liba/0.1@user/testing')})
        client.run("create . libb/0.1@user/testing")

        client.save({"conanfile.py": GenConanfile().with_require('libb/0.1@user/testing')})
        client.run("create . libc/0.1@user/testing")

        client.save({"conanfile.py": GenConanfile().with_require('libc/0.1@user/testing')})
        # Telling to build LibA doesn't change the final result of LibA, which has same ID and PREV
        client.run("install . libd/0.1@user/testing --build=liba")
        # So it is not necessary to build the downstream consumers of LibA
        for lib in ("libb", "libc"):
            self.assertIn("%s/0.1@user/testing: Unknown binary" % lib, client.out)
            self.assertIn("%s/0.1@user/testing: Updated ID" % lib, client.out)
            self.assertIn("%s/0.1@user/testing: Binary for updated ID from: Cache" % lib, client.out)
            self.assertIn("%s/0.1@user/testing: Already installed!" % lib, client.out)

    @pytest.mark.skipif(not get_env("TESTING_REVISIONS_ENABLED", False), reason="Only revisions")
    def test_download_artifacts_after_build(self):
        # An unknown binary that after build results in the exact same PREF with PREV, doesn't
        # fire build of downstream
        client = TestClient(default_server_user=True)
        client.run("config set general.default_package_id_mode=package_revision_mode")
        client.save({"conanfile.py": GenConanfile()})
        client.run("create . liba/0.1@user/testing")
        self.assertIn("liba/0.1@user/testing:5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9 - Build",
                      client.out)
        self.assertIn("liba/0.1@user/testing: Created package revision "
                      "83c38d3b4e5f1b8450434436eec31b00", client.out)
        client.save({"conanfile.py": GenConanfile().with_require('liba/0.1@user/testing')})
        client.run("create . libb/0.1@user/testing")
        self.assertIn("libb/0.1@user/testing:830b7cbbb4fc193a756c82b19904df775dc92204 - Build",
                      client.out)

        client.save({"conanfile.py": GenConanfile().with_require('libb/0.1@user/testing')})
        client.run("create . libc/0.1@user/testing")
        client.run("upload * --all -c")
        client.run("remove * -f")

        client.save({"conanfile.py": GenConanfile().with_require('libc/0.1@user/testing')})
        # Telling to build LibA doesn't change the final result of LibA, which has same ID and PREV
        client.run("install . --build=liba")
        self.assertIn("liba/0.1@user/testing: Created package revision "
                      "83c38d3b4e5f1b8450434436eec31b00", client.out)
        # So it is not necessary to build the downstream consumers of LibA
        for lib in ("libb", "libc"):
            self.assertIn("%s/0.1@user/testing: Unknown binary" % lib, client.out)
            self.assertIn("%s/0.1@user/testing: Updated ID" % lib, client.out)
            self.assertIn("%s/0.1@user/testing: Binary for updated ID from: Download" % lib,
                          client.out)
            self.assertIn("%s/0.1@user/testing: Downloaded package" % lib, client.out)


class PackageRevisionModeTest(unittest.TestCase):

    def setUp(self):
        self.client = TestClient()
        self.client.run("config set general.default_package_id_mode=package_revision_mode")

    def _generate_graph(self, dependencies):
        for ref, deps in dependencies.items():
            ref = ConanFileReference.loads(ref)
            conanfile = GenConanfile().with_name(ref.name).with_version(ref.version)
            for dep in deps:
                conanfile.with_require(ConanFileReference.loads(dep))
            filename = "%s.py" % ref.name
            self.client.save({filename: conanfile})
            self.client.run("export %s %s@" % (filename, ref))

    def test_simple_dependency_graph(self):
        dependencies = {
            "Log4Qt/0.3.0": [],
            "MccApi/3.0.9": ["Log4Qt/0.3.0"],
            "Util/0.3.5": ["MccApi/3.0.9"],
            "Invent/1.0": ["Util/0.3.5"]
        }
        self._generate_graph(dependencies)

        self.client.run("install Invent.py --build missing")
        self.assertIn("MccApi/3.0.9: Package '484784c96c359def1283e7354eec200f9f9c5cd8' created",
                      self.client.out)
        self.assertIn("Util/0.3.5: Package 'ba438cd9d192b914edb1669b3e0149822290f7d8' created",
                      self.client.out)

    def test_triangle_dependency_graph(self):
        dependencies = {
            "Log4Qt/0.3.0": [],
            "MccApi/3.0.9": ["Log4Qt/0.3.0"],
            "Util/0.3.5": ["MccApi/3.0.9"],
            "GenericSU/1.0": ["Log4Qt/0.3.0", "MccApi/3.0.9", "Util/0.3.5"]
                        }
        self._generate_graph(dependencies)

        self.client.run("install GenericSU.py --build missing")
        self.assertIn("MccApi/3.0.9: Package '484784c96c359def1283e7354eec200f9f9c5cd8' created",
                      self.client.out)
        self.assertIn("Util/0.3.5: Package 'ba438cd9d192b914edb1669b3e0149822290f7d8' created",
                      self.client.out)

    def test_diamond_dependency_graph(self):
        dependencies = {
            "Log4Qt/0.3.0": [],
            "MccApi/3.0.9": ["Log4Qt/0.3.0"],
            "Util/0.3.5": ["Log4Qt/0.3.0"],
            "GenericSU/0.3.5": ["MccApi/3.0.9", "Util/0.3.5"]
                        }
        self._generate_graph(dependencies)

        self.client.run("install GenericSU.py --build missing")
        self.assertIn("MccApi/3.0.9: Package '484784c96c359def1283e7354eec200f9f9c5cd8' created",
                      self.client.out)
        self.assertIn("Util/0.3.5: Package '484784c96c359def1283e7354eec200f9f9c5cd8' created",
                      self.client.out)

    def test_full_dependency_graph(self):
        dependencies = {
            "Log4Qt/0.3.0": [],
            "MccApi/3.0.9": ["Log4Qt/0.3.0"],
            "Util/0.3.5": ["MccApi/3.0.9"],
            "GenericSU/0.3.5": ["Log4Qt/0.3.0", "MccApi/3.0.9", "Util/0.3.5"],
            "ManagementModule/0.3.5": ["Log4Qt/0.3.0", "MccApi/3.0.9", "Util/0.3.5"],
            "StationInterfaceModule/0.13.0": ["ManagementModule/0.3.5", "GenericSU/0.3.5"],
            "PleniterGenericSuApp/0.1.8": ["ManagementModule/0.3.5", "GenericSU/0.3.5",
                                           "Log4Qt/0.3.0", "MccApi/3.0.9", "Util/0.3.5"],
            "StationinterfaceRpm/2.2.0": ["StationInterfaceModule/0.13.0",
                                          "PleniterGenericSuApp/0.1.8"]
                        }
        self._generate_graph(dependencies)

        # Obtained with with create and reevaluate_node
        ids = {"Log4Qt/0.3.0": "5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9",
               "MccApi/3.0.9": "484784c96c359def1283e7354eec200f9f9c5cd8",
               "Util/0.3.5": "ba438cd9d192b914edb1669b3e0149822290f7d8",
               "GenericSU/0.3.5": "c152f6964d097021edab4a508d8fa926bad976fb",
               "ManagementModule/0.3.5": "c152f6964d097021edab4a508d8fa926bad976fb",
               "StationInterfaceModule/0.13.0": "7184518aa4cb352204d8b00d5424d3e52e5819d8",
               "PleniterGenericSuApp/0.1.8": "7184518aa4cb352204d8b00d5424d3e52e5819d8"}

        rev = {"Log4Qt/0.3.0": "ce3408b2884c458b2bcdfeff92404c85",
               "MccApi/3.0.9": "45aeac67977b1509a2d02f9a205baccb",
               "Util/0.3.5": "f446fb8e24603baafacda66aadd2503f",
               "GenericSU/0.3.5": "d5761d4d690ded2003c3cdab86e55d15",
               "ManagementModule/0.3.5": "743935f4f5664b059d54d361a232bf94",
               "StationInterfaceModule/0.13.0": "fc81fa83c3c134db6ba5af99fd8460cb",
               "PleniterGenericSuApp/0.1.8": "7191e6b6eaf79194f1b083c8dc508518"}

        self.client.run("install StationinterfaceRpm.py --build missing")
        for pkg, id_ in ids.items():
            self.assertIn("%s: Package '%s' created" % (pkg, id_), self.client.out)
        for pkg, r in rev.items():
            self.assertIn("%s: Created package revision %s" % (pkg, r), self.client.out)

        self.client.run("install StationinterfaceRpm.py")
        for pkg, id_ in ids.items():
            self.assertIn("%s:%s - Cache" % (pkg, id_), self.client.out)


def test_package_revision_mode_full_transitive_package_id():
    # https://github.com/conan-io/conan/issues/8310
    client = TestClient()
    client.run("config set general.default_package_id_mode=package_revision_mode")
    client.run("config set general.full_transitive_package_id=1")
    client.save({"tool/conanfile.py": GenConanfile(),
                 "pkga/conanfile.py": GenConanfile(),
                 "pkgb/conanfile.py": GenConanfile().with_requires("tool/0.1", "pkga/0.1"),
                 "profile": "[build_requires]\ntool/0.1"})
    client.run("export tool tool/0.1@")
    client.run("export pkga pkga/0.1@")
    client.run("create pkgb pkgb/0.1@ -pr=profile --build=missing")
    assert "pkgb/0.1:Package_ID_unknown - Unknown" in client.out
    assert "pkgb/0.1: Unknown binary for pkgb/0.1, computing updated ID" in client.out
    assert "pkgb/0.1: Package 'f524f1981a44932e1445e13ae4cf9e2ff8112027' created" in client.out



import textwrap

import pytest

from conan.tools.microsoft import unix_path
from conans.model.conf import ConfDefinition
from conans.test.utils.mocks import MockSettings, ConanFileMock


@pytest.mark.parametrize("subsystem, expected_path", [
    ("msys2", '/c/path/to/stuff'),
    ("msys", '/c/path/to/stuff'),
    ("cygwin", '/cygdrive/c/path/to/stuff'),
    ("wsl", '/mnt/c/path/to/stuff'),
    ("sfu", '/dev/fs/C/path/to/stuff')
])
def test_unix_path(subsystem, expected_path):
    c = ConfDefinition()
    c.loads(textwrap.dedent("""\
        tools.microsoft.bash:subsystem={}
    """.format(subsystem)))

    settings = MockSettings({"os": "Windows"})
    conanfile = ConanFileMock()
    conanfile.conf = c.get_conanfile_conf(None)
    conanfile.settings = settings
    conanfile.settings_build = settings
    conanfile.win_bash = True

    path = unix_path(conanfile, "c:/path/to/stuff")
    assert expected_path == path



import platform
import textwrap

import pytest
import six

from conans.test.utils.tools import TestClient


@pytest.mark.tool_apt_get
@pytest.mark.skipif(platform.system() != "Linux", reason="Requires apt")
def test_apt_check():
    client = TestClient()
    client.save({"conanfile.py": textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.system.package_manager import Apt
        class MyPkg(ConanFile):
            settings = "arch", "os"
            def system_requirements(self):
                apt = Apt(self)
                not_installed = apt.check(["non-existing1", "non-existing2"])
                print("missing:", not_installed)
        """)})
    client.run("create . test/1.0@ -s:b arch=armv8 -s:h arch=x86")
    assert "dpkg-query: no packages found matching non-existing1:i386" in client.out
    assert "dpkg-query: no packages found matching non-existing2:i386" in client.out
    assert "missing: ['non-existing1', 'non-existing2']" in client.out


@pytest.mark.tool_apt_get
@pytest.mark.skipif(platform.system() != "Linux", reason="Requires apt")
def test_apt_install_substitutes():
    client = TestClient()
    conanfile_py = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.system.package_manager import Apt
        class MyPkg(ConanFile):
            settings = "arch", "os"
            def system_requirements(self):
                # FIXME this is needed because the ci-functional apt-get update fails
                try:
                    self.run("sudo apt-get update")
                except Exception:
                    pass
                apt = Apt(self)
                {}
        """)

    installs = 'apt.install_substitutes(["non-existing1", "non-existing2"], ["non-existing3", "non-existing4"])'
    client.save({"conanfile.py": conanfile_py.format(installs)})
    client.run("create . test/1.0@ -c tools.system.package_manager:mode=install "
               "-c tools.system.package_manager:sudo=True", assert_error=True)

    assert "dpkg-query: no packages found matching non-existing1:amd64" in client.out
    assert "dpkg-query: no packages found matching non-existing2:amd64" in client.out
    assert "dpkg-query: no packages found matching non-existing3:amd64" in client.out
    assert "dpkg-query: no packages found matching non-existing4:amd64" in client.out
    assert "ERROR: while executing system_requirements(): " \
           "None of the installs for the package substitutes succeeded." in client.out

    client.run_command("sudo apt remove nano -yy")
    installs = 'apt.install_substitutes(["non-existing1", "non-existing2"], ["nano"], ["non-existing3"])'
    client.save({"conanfile.py": conanfile_py.format(installs)})
    client.run("create . test/1.0@ -c tools.system.package_manager:mode=install "
               "-c tools.system.package_manager:sudo=True")
    assert "1 newly installed" in client.out


@pytest.mark.tool_apt_get
@pytest.mark.skipif(platform.system() != "Linux", reason="Requires apt")
def test_build_require():
    client = TestClient()
    client.save({"tool_require.py": textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.system.package_manager import Apt
        class MyPkg(ConanFile):
            settings = "arch", "os"
            def system_requirements(self):
                apt = Apt(self)
                not_installed = apt.check(["non-existing1", "non-existing2"])
                print("missing:", not_installed)
        """)})
    client.run("export tool_require.py tool_require/1.0@")
    client.save({"consumer.py": textwrap.dedent("""
        from conan import ConanFile
        class consumer(ConanFile):
            settings = "arch", "os"
            tool_requires = "tool_require/1.0"
        """)})
    client.run("create consumer.py consumer/1.0@ -s:b arch=armv8 -s:h arch=x86 --build=missing")
    assert "dpkg-query: no packages found matching non-existing1:arm64" in client.out
    assert "dpkg-query: no packages found matching non-existing2:arm64" in client.out
    assert "missing: ['non-existing1', 'non-existing2']" in client.out


@pytest.mark.tool_brew
@pytest.mark.skipif(platform.system() != "Darwin", reason="Requires brew")
def test_brew_check():
    client = TestClient()
    client.save({"conanfile.py": textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.system.package_manager import Brew
        class MyPkg(ConanFile):
            settings = "arch"
            def system_requirements(self):
                brew = Brew(self)
                not_installed = brew.check(["non-existing1", "non-existing2"])
                print("missing:", not_installed)
        """)})
    client.run("create . test/1.0@")
    assert "missing: ['non-existing1', 'non-existing2']" in client.out


@pytest.mark.tool_brew
@pytest.mark.skipif(platform.system() != "Darwin", reason="Requires brew")
@pytest.mark.skip(reason="brew update takes a lot of time")
def test_brew_install_check_mode():
    client = TestClient()
    client.save({"conanfile.py": textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.system.package_manager import Brew
        class MyPkg(ConanFile):
            settings = "arch"
            def system_requirements(self):
                brew = Brew(self)
                brew.install(["non-existing1", "non-existing2"])
        """)})
    client.run("create . test/1.0@", assert_error=True)
    assert "System requirements: 'non-existing1, non-existing2' are missing but " \
           "can't install because tools.system.package_manager:mode is 'check'" in client.out


@pytest.mark.tool_brew
@pytest.mark.skipif(platform.system() != "Darwin", reason="Requires brew")
@pytest.mark.skip(reason="brew update takes a lot of time")
def test_brew_install_install_mode():
    client = TestClient()
    client.save({"conanfile.py": textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.system.package_manager import Brew
        class MyPkg(ConanFile):
            settings = "arch"
            def system_requirements(self):
                brew = Brew(self)
                brew.install(["non-existing1", "non-existing2"])
        """)})
    client.run("create . test/1.0@ -c tools.system.package_manager:mode=install", assert_error=True)
    assert "Error: No formulae found in taps." in client.out



# coding=utf-8


import unittest
from collections import namedtuple

import six
from mock import mock

from conans.client.cmd.export import _update_revision_in_metadata
from conans.model.ref import ConanFileReference
from conans.paths.package_layouts.package_cache_layout import PackageCacheLayout
from conans.test.utils.test_files import temp_folder
from conans.errors import ConanException
from conans.test.utils.mocks import TestBufferConanOutput


class UpdateRevisionInMetadataTests(unittest.TestCase):

    def setUp(self):
        ref = ConanFileReference.loads("lib/version@user/channel")
        self.package_layout = PackageCacheLayout(base_folder=temp_folder(), ref=ref,
                                                 short_paths=False, no_lock=True)
        self.output = TestBufferConanOutput()

    def test_scm_warn_not_pristine(self):
        with mock.patch("conans.client.cmd.export._detect_scm_revision",
                        return_value=("revision", "git", False)):
            path = None
            digest = namedtuple("Digest", "summary_hash")
            _update_revision_in_metadata(self.package_layout, True, self.output,
                                         path, digest, "scm")
            self.assertIn("WARN: Repo status is not pristine: there might be modified files",
                          self.output)

    def test_scm_behavior(self):
        revision_mode = "scm"

        digest = None
        path = None
        with mock.patch("conans.client.cmd.export._detect_scm_revision",
                        return_value=("1234", "git", True)):
            rev = _update_revision_in_metadata(self.package_layout, True, self.output,
                                               path, digest, revision_mode)
        self.assertEqual(rev, "1234")
        self.assertIn("Using git commit as the recipe revision", self.output)

    def test_hash_behavior(self):
        revision_mode = "hash"

        digest = namedtuple("Digest", "summary_hash")
        digest.summary_hash = "1234"
        path = None
        rev = _update_revision_in_metadata(self.package_layout, True, self.output,
                                           path, digest, revision_mode)
        self.assertEqual(rev, "1234")
        self.assertIn("Using the exported files summary hash as the recipe revision", self.output)

    def test_invalid_behavior(self):
        revision_mode = "auto"
        digest = path = None

        with six.assertRaisesRegex(self, ConanException, "Revision mode should be"):
            _update_revision_in_metadata(self.package_layout, True, self.output,
                                           path, digest, revision_mode)



import os
import unittest
import platform
import subprocess
import json

import pytest

from conans.client import tools
from conans.test.utils.profiles import create_profile
from conans.client.conf.detect import detect_defaults_settings
from conans.test.utils.tools import TestClient
from conans.util.files import load
from conans.util.runners import check_output_runner
from conans.paths import DEFAULT_PROFILE_NAME
from conans.test.utils.mocks import TestBufferConanOutput


class ProfileTest(unittest.TestCase):

    def test_reuse_output(self):
        client = TestClient()
        client.run("profile new myprofile --detect")
        client.run("profile update options.Pkg:myoption=123 myprofile")
        client.run("profile update env.Pkg2:myenv=123 myprofile")
        client.run("profile update conf.tools.ninja:jobs=10 myprofile")
        client.run("profile show myprofile")
        self.assertIn("Pkg:myoption=123", client.out)
        self.assertIn("Pkg2:myenv=123", client.out)
        self.assertIn("tools.ninja:jobs=10", client.out)
        profile = str(client.out).splitlines()[2:]
        client.save({"conanfile.txt": "",
                     "mylocalprofile": "\n".join(profile)})
        client.run("install . -pr=mylocalprofile")
        self.assertIn("conanfile.txt: Generated conaninfo.txt", client.out)

    def test_empty(self):
        client = TestClient(cache_autopopulate=False)
        client.run("profile list")
        self.assertIn("No profiles defined", client.out)

    def test_list(self):
        client = TestClient()
        profiles = ["default", "profile1", "profile2", "profile3",
                    "nested" + os.path.sep + "profile4",
                    "nested" + os.path.sep + "two" + os.path.sep + "profile5",
                    "nested" + os.path.sep + "profile6"]
        if platform.system() != "Windows":
            profiles.append("symlink_me" + os.path.sep + "profile7")
        for profile in profiles:
            create_profile(client.cache.profiles_path, profile)

        if platform.system() != "Windows":
            os.symlink(os.path.join(client.cache.profiles_path, 'symlink_me'),
                       os.path.join(client.cache.profiles_path, 'link'))
            # profile7 will be shown twice because it is symlinked.
            profiles.append("link" + os.path.sep + "profile7")

        # Make sure local folder doesn't interact with profiles
        os.mkdir(os.path.join(client.current_folder, "profile3"))
        client.run("profile list")
        profiles.sort()
        self.assertEqual(profiles, list(str(client.out).splitlines()))

        # Test profile list json file
        client.run("profile list --json profile_list.json")
        json_path = os.path.join(client.current_folder, "profile_list.json")
        self.assertTrue(os.path.exists(json_path))
        json_content = load(json_path)
        json_obj = json.loads(json_content)
        self.assertEqual(list, type(json_obj))
        self.assertEqual(profiles, json_obj)

    def test_show(self):
        client = TestClient()
        create_profile(client.cache.profiles_path, "profile1", settings={"os": "Windows"},
                       options=[("MyOption", "32")])
        create_profile(client.cache.profiles_path, "profile3",
                       env=[("package:VAR", "value"), ("CXX", "/path/tomy/g++_build"),
                            ("CC", "/path/tomy/gcc_build")],
                       conf=["tools.ninja:jobs=10", "tools.gnu.make:jobs=20"])
        client.run("profile show profile1")
        self.assertIn("[settings]\nos=Windows", client.out)
        self.assertIn("MyOption=32", client.out)
        client.run("profile show profile3")
        self.assertIn("CC=/path/tomy/gcc_build", client.out)
        self.assertIn("CXX=/path/tomy/g++_build", client.out)
        self.assertIn("package:VAR=value", client.out)
        self.assertIn("tools.ninja:jobs=10", client.out)
        self.assertIn("tools.gnu.make:jobs=20", client.out)

    def test_profile_update_and_get(self):
        client = TestClient()
        client.run("profile new ./MyProfile --detect")
        if "WARNING: GCC OLD ABI COMPATIBILITY" in client.out:
            self.assertIn("Or edit '{}/MyProfile' and "
                          "set compiler.libcxx=libstdc++11".format(client.current_folder),
                          client.out)

        pr_path = os.path.join(client.current_folder, "MyProfile")

        client.run("profile update settings.os=FakeOS ./MyProfile")
        self.assertIn("\nos=FakeOS", load(pr_path))
        self.assertNotIn("\nos=Linux", load(pr_path))

        client.run("profile get settings.os ./MyProfile")
        self.assertEqual(client.out, "FakeOS\n")

        client.run("profile update settings.compiler.version=88 ./MyProfile")
        self.assertIn("compiler.version=88", load(pr_path))

        client.run("profile get settings.compiler.version ./MyProfile")
        self.assertEqual(client.out, "88\n")

        client.run("profile update options.MyOption=23 ./MyProfile")
        self.assertIn("[options]\nMyOption=23", load(pr_path))

        client.run("profile get options.MyOption ./MyProfile")
        self.assertEqual(client.out, "23\n")

        client.run("profile update options.Package:MyOption=23 ./MyProfile")
        self.assertIn("Package:MyOption=23", load(pr_path))

        client.run("profile get options.Package:MyOption ./MyProfile")
        self.assertEqual(client.out, "23\n")

        client.run("profile update options.Package:OtherOption=23 ./MyProfile")
        self.assertIn("Package:OtherOption=23", load(pr_path))

        client.run("profile get options.Package:OtherOption ./MyProfile")
        self.assertEqual(client.out, "23\n")

        client.run("profile update env.OneMyEnv=MYVALUe ./MyProfile")
        self.assertIn("[env]\nOneMyEnv=MYVALUe", load(pr_path))

        client.run("profile get env.OneMyEnv ./MyProfile")
        self.assertEqual(client.out, "MYVALUe\n")

        client.run("profile update conf.tools.ninja:jobs=10 ./MyProfile")
        self.assertIn("[conf]\ntools.ninja:jobs=10", load(pr_path))

        client.run("profile get conf.tools.ninja:jobs ./MyProfile")
        self.assertEqual(client.out, "10\n")

        client.run("profile update conf.tools.gnu.make:jobs=20 ./MyProfile")
        self.assertIn("tools.gnu.make:jobs=20", load(pr_path))

        client.run("profile get conf.tools.gnu.make:jobs ./MyProfile")
        self.assertEqual(client.out, "20\n")

        # Now try the remove

        client.run("profile remove settings.os ./MyProfile")
        client.run("profile remove settings.os_build ./MyProfile")
        self.assertNotIn("os=", load(pr_path))

        client.run("profile remove settings.compiler.version ./MyProfile")
        self.assertNotIn("compiler.version", load(pr_path))

        client.run("profile remove options.MyOption ./MyProfile")
        self.assertNotIn("[options]\nMyOption", load(pr_path))

        client.run("profile remove options.Package:MyOption ./MyProfile")
        self.assertNotIn("Package:MyOption", load(pr_path))
        self.assertIn("Package:OtherOption", load(pr_path))

        client.run("profile remove conf.tools.gnu.make:jobs ./MyProfile")
        self.assertNotIn("tools.gnu.make:jobs", load(pr_path))
        self.assertIn("tools.ninja:jobs", load(pr_path))

        client.run("profile remove env.OneMyEnv ./MyProfile")
        self.assertNotIn("OneMyEnv", load(pr_path))

        # Remove a non existent key
        client.run("profile remove settings.os ./MyProfile", assert_error=True)
        self.assertIn("Profile key 'settings.os' doesn't exist", client.out)

        client.run("profile remove options.foo ./MyProfile", assert_error=True)
        self.assertIn("Profile key 'options.foo' doesn't exist", client.out)

        client.run("profile remove env.foo ./MyProfile", assert_error=True)
        self.assertIn("Profile key 'env.foo' doesn't exist", client.out)

        client.run("profile remove conf.MyConf ./MyProfile", assert_error=True)
        self.assertIn("Profile key 'conf.MyConf' doesn't exist", client.out)

    def test_profile_update_env(self):
        client = TestClient()
        client.run("profile new ./MyProfile")
        pr_path = os.path.join(client.current_folder, "MyProfile")

        client.run("profile update env.foo=bar ./MyProfile")
        self.assertEqual(["[env]", "foo=bar"], load(pr_path).splitlines()[-2:])
        client.run("profile update env.foo=BAZ ./MyProfile")
        self.assertEqual(["[env]", "foo=BAZ"], load(pr_path).splitlines()[-2:])
        client.run("profile update env.MyPkg:foo=FOO ./MyProfile")
        self.assertEqual(["[env]", "foo=BAZ", "MyPkg:foo=FOO"], load(pr_path).splitlines()[-3:])
        client.run("profile update env.MyPkg:foo=FOO,BAZ,BAR ./MyProfile")
        self.assertEqual(["[env]", "foo=BAZ", "MyPkg:foo=FOO,BAZ,BAR"],
                         load(pr_path).splitlines()[-3:])

    def test_profile_new(self):
        client = TestClient()
        client.run("profile new ./MyProfile")
        pr_path = os.path.join(client.current_folder, "MyProfile")
        self.assertTrue(os.path.exists(pr_path))
        self.assertEqual(load(pr_path), """[settings]
[options]
[build_requires]
[env]
""")

        client.run("profile new ./MyProfile2 --detect")
        pr_path = os.path.join(client.current_folder, "MyProfile2")
        self.assertTrue(os.path.exists(os.path.join(client.current_folder, "MyProfile2")))
        self.assertIn("os=", load(pr_path))

        client.run("profile new ./MyProfile2 --detect", assert_error=True)
        self.assertIn("Profile already exists", client.out)

        client.run("profile new MyProfile3")
        pr_path = os.path.join(client.cache.profiles_path, "MyProfile3")
        self.assertTrue(os.path.exists(pr_path))
        self.assertNotIn("os=", load(pr_path))

    def test_profile_force_new(self):
        client = TestClient()

        empty_profile = """[settings]
[options]
[build_requires]
[env]
"""
        client.run("profile new ./MyProfile")
        pr_path = os.path.join(client.current_folder, "MyProfile")
        self.assertTrue(os.path.exists(pr_path))
        self.assertEqual(load(pr_path), empty_profile)

        client.run("profile new ./MyProfile --detect", assert_error=True)
        self.assertIn("Profile already exists", client.out)

        client.run("profile new ./MyProfile --detect --force", assert_error=False)
        self.assertNotEqual(load(pr_path), empty_profile)

        detected_profile = load(pr_path)

        client.run("profile update settings.os=FakeOS ./MyProfile")
        self.assertIn("\nos=FakeOS", load(pr_path))

        client.run("profile update env.MyEnv=MYVALUe ./MyProfile")
        self.assertIn("[env]\nMyEnv=MYVALUe", load(pr_path))

        client.run("profile new ./MyProfile --detect --force", assert_error=False)
        self.assertNotIn("\nos=FakeOS", load(pr_path))
        self.assertNotIn("[env]\nMyEnv=MYVALUe", load(pr_path))
        self.assertEqual(load(pr_path), detected_profile)

    def test_missing_subarguments(self):
        client = TestClient()
        client.run("profile", assert_error=True)
        self.assertIn("ERROR: Exiting with code: 2", client.out)


class DetectCompilersTest(unittest.TestCase):
    def test_detect_default_compilers(self):
        platform_default_compilers = {
            "Linux": "gcc",
            "Darwin": "apple-clang",
            "Windows": "Visual Studio"
        }
        output = TestBufferConanOutput()
        result = detect_defaults_settings(output, profile_path=DEFAULT_PROFILE_NAME)
        # result is a list of tuples (name, value) so converting it to dict
        result = dict(result)
        platform_compiler = platform_default_compilers.get(platform.system(), None)
        if platform_compiler is not None:
            self.assertEqual(result.get("compiler", None), platform_compiler)

    @pytest.mark.tool_gcc
    @pytest.mark.skipif(platform.system() != "Darwin", reason="only OSX test")
    def test_detect_default_in_mac_os_using_gcc_as_default(self):
        """
        Test if gcc in Mac OS X is using apple-clang as frontend
        """
        # See: https://github.com/conan-io/conan/issues/2231
        output = check_output_runner(["gcc", "--version"], stderr=subprocess.STDOUT)

        if "clang" not in output:
            # Not test scenario gcc should display clang in output
            # see: https://stackoverflow.com/questions/19535422/os-x-10-9-gcc-links-to-clang
            raise Exception("Apple gcc doesn't point to clang with gcc frontend anymore!")

        output = TestBufferConanOutput()
        with tools.environment_append({"CC": "gcc"}):
            result = detect_defaults_settings(output, profile_path=DEFAULT_PROFILE_NAME)
        # result is a list of tuples (name, value) so converting it to dict
        result = dict(result)
        # No compiler should be detected
        self.assertIsNone(result.get("compiler", None))
        self.assertIn("gcc detected as a frontend using apple-clang", output)
        self.assertIsNotNone(output.error)



import mock
import platform
import six
import unittest
from six import StringIO

from conans import tools
from conans.client.output import ConanOutput
from conans.client.tools.files import which
from conans.client.tools.oss import OSInfo
from conans.client.tools.system_pm import ChocolateyTool, SystemPackageTool, AptTool
from conans.errors import ConanException, ConanInvalidSystemRequirements
from conans.test.unittests.util.tools_test import RunnerMock
from conans.test.utils.mocks import MockSettings, MockConanfile, TestBufferConanOutput


class RunnerMultipleMock(object):
    def __init__(self, expected=None):
        self.calls = 0
        self.expected = expected

    def __call__(self, command, *args, **kwargs):  # @UnusedVariable
        self.calls += 1
        return 0 if command in self.expected else 1


class SystemPackageToolTest(unittest.TestCase):
    def setUp(self):
        self.out = TestBufferConanOutput()

    def test_sudo_tty(self):
        with tools.environment_append({"CONAN_SYSREQUIRES_SUDO": "False"}):
            self.assertFalse(SystemPackageTool._is_sudo_enabled())
            self.assertEqual(SystemPackageTool._get_sudo_str(), "")

        with tools.environment_append({"CONAN_SYSREQUIRES_SUDO": "True"}):
            self.assertTrue(SystemPackageTool._is_sudo_enabled())
            self.assertEqual(SystemPackageTool._get_sudo_str(), "sudo -A ")

            with mock.patch("sys.stdout.isatty", return_value=True):
                self.assertEqual(SystemPackageTool._get_sudo_str(), "sudo ")

    def test_system_without_sudo(self):
        with mock.patch("os.path.isfile", return_value=False):
            self.assertFalse(SystemPackageTool._is_sudo_enabled())
            self.assertEqual(SystemPackageTool._get_sudo_str(), "")

            with mock.patch("sys.stdout.isatty", return_value=True):
                self.assertEqual(SystemPackageTool._get_sudo_str(), "")

    def test_verify_update(self):
        # https://github.com/conan-io/conan/issues/3142
        with tools.environment_append({"CONAN_SYSREQUIRES_SUDO": "False",
                                       "CONAN_SYSREQUIRES_MODE": "Verify"}):
            runner = RunnerMock()
            # fake os info to linux debian, default sudo
            os_info = OSInfo()
            os_info.is_macos = False
            os_info.is_linux = True
            os_info.is_windows = False
            os_info.linux_distro = "debian"
            spt = SystemPackageTool(runner=runner, os_info=os_info, output=self.out)
            spt.update()
            self.assertEqual(runner.command_called, None)
            self.assertIn('Not updating system_requirements. CONAN_SYSREQUIRES_MODE=verify',
                          self.out)

    # We gotta mock the with_apt property, since it checks for the existence of apt.
    @mock.patch('conans.client.tools.oss.OSInfo.with_apt', new_callable=mock.PropertyMock)
    def test_add_repositories_exception_cases(self, patched_with_apt):
        os_info = OSInfo()
        os_info.is_macos = False
        os_info.is_linux = True
        os_info.is_windows = False
        os_info.linux_distro = "fedora"  # Will instantiate DnfTool

        patched_with_apt.return_value = False

        with six.assertRaisesRegex(self, ConanException, "add_repository not implemented"):
            new_out = StringIO()
            spt = SystemPackageTool(os_info=os_info, output=ConanOutput(new_out))
            spt.add_repository(repository="deb http://repo/url/ saucy universe multiverse",
                               repo_key=None)

    @mock.patch('conans.client.tools.oss.OSInfo.with_apt', new_callable=mock.PropertyMock)
    def test_add_repository(self, patched_with_apt):
        class RunnerOrderedMock:
            commands = []  # Command + return value

            def __call__(runner_self, command, output, win_bash=False, subsystem=None):
                if not len(runner_self.commands):
                    self.fail("Commands list exhausted, but runner called with '%s'" % command)
                expected, ret = runner_self.commands.pop(0)
                self.assertEqual(expected, command)
                return ret

        def _run_add_repository_test(repository, gpg_key, sudo, isatty, update):
            sudo_cmd = ""
            if sudo:
                sudo_cmd = "sudo " if isatty else "sudo -A "

            runner = RunnerOrderedMock()
            if gpg_key:
                runner.commands.append(
                    ("wget -qO - {} | {}apt-key add -".format(gpg_key, sudo_cmd), 0))
            runner.commands.append(("{}apt-add-repository {}".format(sudo_cmd, repository), 0))
            if update:
                runner.commands.append(("{}apt-get update".format(sudo_cmd), 0))

            with tools.environment_append({"CONAN_SYSREQUIRES_SUDO": str(sudo)}):
                os_info = OSInfo()
                os_info.is_macos = False
                os_info.is_linux = True
                os_info.is_windows = False
                os_info.linux_distro = "debian"
                patched_with_apt.return_value = True

                new_out = StringIO()
                spt = SystemPackageTool(runner=runner, os_info=os_info, output=ConanOutput(new_out))

                spt.add_repository(repository=repository, repo_key=gpg_key, update=update)
                self.assertEqual(len(runner.commands), 0)

        # Run several test cases
        repository = "deb http://repo/url/ saucy universe multiverse"
        gpg_key = 'http://one/key.gpg'
        _run_add_repository_test(repository, gpg_key, sudo=True, isatty=False, update=True)
        _run_add_repository_test(repository, gpg_key, sudo=True, isatty=False, update=False)
        _run_add_repository_test(repository, gpg_key, sudo=False, isatty=False, update=True)
        _run_add_repository_test(repository, gpg_key, sudo=False, isatty=False, update=False)
        _run_add_repository_test(repository, gpg_key=None, sudo=True, isatty=False, update=True)
        _run_add_repository_test(repository, gpg_key=None, sudo=False, isatty=True, update=False)

        with mock.patch("sys.stdout.isatty", return_value=True):
            _run_add_repository_test(repository, gpg_key, sudo=True, isatty=True, update=True)

    # We gotta mock the with_apt property, since it checks for the existence of apt.
    @mock.patch('conans.client.tools.oss.OSInfo.with_apt', new_callable=mock.PropertyMock)
    def test_system_package_tool(self, patched_with_apt):

        with tools.environment_append({"CONAN_SYSREQUIRES_SUDO": "True"}):
            runner = RunnerMock()
            # fake os info to linux debian, default sudo
            os_info = OSInfo()
            os_info.is_macos = False
            os_info.is_linux = True
            os_info.is_windows = False
            patched_with_apt.return_value = True

            os_info.linux_distro = "debian"

            spt = SystemPackageTool(runner=runner, os_info=os_info, output=self.out)
            spt.update()
            self.assertEqual(runner.command_called, "sudo -A apt-get update")

            os_info.linux_distro = "ubuntu"
            spt = SystemPackageTool(runner=runner, os_info=os_info, output=self.out)
            spt.update()
            self.assertEqual(runner.command_called, "sudo -A apt-get update")

            os_info.linux_distro = "knoppix"
            spt = SystemPackageTool(runner=runner, os_info=os_info, output=self.out)
            spt.update()
            self.assertEqual(runner.command_called, "sudo -A apt-get update")

            os_info.linux_distro = "neon"
            spt = SystemPackageTool(runner=runner, os_info=os_info, output=self.out)
            spt.update()
            self.assertEqual(runner.command_called, "sudo -A apt-get update")

            # We'll be testing non-Ubuntu and non-Debian-based distros.
            patched_with_apt.return_value = False

            with mock.patch("conans.client.tools.oss.which", return_value=True):
                os_info.linux_distro = "fedora"
                spt = SystemPackageTool(runner=runner, os_info=os_info, output=self.out)
                spt.update()
                self.assertEqual(runner.command_called, "sudo -A dnf check-update -y")

            # Without DNF in the path,
            os_info.linux_distro = "fedora"
            spt = SystemPackageTool(runner=runner, os_info=os_info, output=self.out)
            spt.update()
            self.assertEqual(runner.command_called, "sudo -A yum check-update -y")

            os_info.linux_distro = "opensuse"
            spt = SystemPackageTool(runner=runner, os_info=os_info, output=self.out)
            spt.update()
            self.assertEqual(runner.command_called, "sudo -A zypper --non-interactive ref")

            os_info.linux_distro = "redhat"
            spt = SystemPackageTool(runner=runner, os_info=os_info, output=self.out)
            spt.install("a_package", force=False)
            self.assertEqual(runner.command_called, "rpm -q a_package")
            spt.install("a_package", force=True)
            self.assertEqual(runner.command_called, "sudo -A yum install -y a_package")

            settings = MockSettings({"arch": "x86", "arch_build": "x86_64", "os": "Linux",
                                     "os_build": "Linux"})
            conanfile = MockConanfile(settings)
            spt = SystemPackageTool(runner=runner, os_info=os_info, output=self.out,
                                    conanfile=conanfile)
            spt.install("a_package", force=False)
            self.assertEqual(runner.command_called, "rpm -q a_package.i?86")
            spt.install("a_package", force=True)
            self.assertEqual(runner.command_called, "sudo -A yum install -y a_package.i?86")

            os_info.linux_distro = "debian"
            patched_with_apt.return_value = True
            spt = SystemPackageTool(runner=runner, os_info=os_info, output=self.out)
            with self.assertRaises(ConanException):
                runner.return_ok = False
                spt.install("a_package")
                self.assertEqual(runner.command_called,
                                 "sudo -A apt-get install -y --no-install-recommends a_package")

            runner.return_ok = True
            spt.install("a_package", force=False)
            self.assertEqual(runner.command_called,
                             'dpkg-query -W -f=\'${Status}\' a_package | grep -q "ok installed"')

            os_info.is_macos = True
            os_info.is_linux = False
            os_info.is_windows = False
            patched_with_apt.return_value = False

            spt = SystemPackageTool(runner=runner, os_info=os_info, output=self.out)
            spt.update()
            self.assertEqual(runner.command_called, "brew update")
            spt.install("a_package", force=True)
            self.assertEqual(runner.command_called, "brew install a_package")

            os_info.is_freebsd = True
            os_info.is_macos = False
            patched_with_apt.return_value = False

            spt = SystemPackageTool(runner=runner, os_info=os_info, output=self.out)
            spt.update()
            self.assertEqual(runner.command_called, "sudo -A pkg update")
            spt.install("a_package", force=True)
            self.assertEqual(runner.command_called, "sudo -A pkg install -y a_package")
            spt.install("a_package", force=False)
            self.assertEqual(runner.command_called, "pkg info a_package")

            # Chocolatey is an optional package manager on Windows
            if platform.system() == "Windows" and which("choco.exe"):
                os_info.is_freebsd = False
                os_info.is_windows = True
                patched_with_apt.return_value = False
                spt = SystemPackageTool(runner=runner, os_info=os_info, output=self.out,
                                        tool=ChocolateyTool(output=self.out))
                spt.update()
                self.assertEqual(runner.command_called, "choco outdated")
                spt.install("a_package", force=True)
                self.assertEqual(runner.command_called, "choco install --yes a_package")
                spt.install("a_package", force=False)
                self.assertEqual(runner.command_called,
                                 'choco search --local-only --exact a_package | '
                                 'findstr /c:"1 packages installed."')

        with tools.environment_append({"CONAN_SYSREQUIRES_SUDO": "False"}):

            os_info = OSInfo()
            os_info.is_linux = True
            os_info.linux_distro = "redhat"
            patched_with_apt.return_value = False

            spt = SystemPackageTool(runner=runner, os_info=os_info, output=self.out)
            spt.install("a_package", force=True)
            self.assertEqual(runner.command_called, "yum install -y a_package")
            spt.update()
            self.assertEqual(runner.command_called, "yum check-update -y")

            os_info.linux_distro = "ubuntu"
            patched_with_apt.return_value = True
            spt = SystemPackageTool(runner=runner, os_info=os_info, output=self.out)
            spt.install("a_package", force=True)
            self.assertEqual(runner.command_called,
                             "apt-get install -y --no-install-recommends a_package")

            spt.update()
            self.assertEqual(runner.command_called, "apt-get update")

            for arch, distro_arch in {"x86_64": "", "x86": ":i386", "ppc32": ":powerpc",
                                      "ppc64le": ":ppc64el", "armv7": ":arm", "armv7hf": ":armhf",
                                      "armv8": ":arm64", "s390x": ":s390x"}.items():
                settings = MockSettings({"arch": arch,
                                         "arch_build": "x86_64",
                                         "os": "Linux",
                                         "os_build": "Linux"})
                conanfile = MockConanfile(settings)
                spt = SystemPackageTool(runner=runner, os_info=os_info, output=self.out,
                                        conanfile=conanfile)
                spt.install("a_package", force=True)
                self.assertEqual(runner.command_called,
                            "apt-get install -y --no-install-recommends a_package%s" % distro_arch)

            for arch, distro_arch in {"x86_64": "", "x86": ":all"}.items():
                settings = MockSettings({"arch": arch,
                                         "arch_build": "x86_64",
                                         "os": "Linux",
                                         "os_build": "Linux"})
                conanfile = MockConanfile(settings)
                spt = SystemPackageTool(runner=runner, os_info=os_info, output=self.out,
                                        conanfile=conanfile)
                spt.install("a_package", force=True, arch_names={"x86": "all"})
                self.assertEqual(runner.command_called,
                            "apt-get install -y --no-install-recommends a_package%s" % distro_arch)

            os_info.is_macos = True
            os_info.is_linux = False
            os_info.is_windows = False
            patched_with_apt.return_value = False
            spt = SystemPackageTool(runner=runner, os_info=os_info, output=self.out)

            spt.update()
            self.assertEqual(runner.command_called, "brew update")
            spt.install("a_package", force=True)
            self.assertEqual(runner.command_called, "brew install a_package")

            os_info.is_freebsd = True
            os_info.is_macos = False
            os_info.is_windows = False
            patched_with_apt.return_value = False

            spt = SystemPackageTool(runner=runner, os_info=os_info, output=self.out)
            spt.update()
            self.assertEqual(runner.command_called, "pkg update")
            spt.install("a_package", force=True)
            self.assertEqual(runner.command_called, "pkg install -y a_package")
            spt.install("a_package", force=False)
            self.assertEqual(runner.command_called, "pkg info a_package")

            os_info.is_solaris = True
            os_info.is_freebsd = False
            os_info.is_windows = False
            patched_with_apt.return_value = False

            spt = SystemPackageTool(runner=runner, os_info=os_info, output=self.out)
            spt.update()
            self.assertEqual(runner.command_called, "pkgutil --catalog")
            spt.install("a_package", force=True)
            self.assertEqual(runner.command_called, "pkgutil --install --yes a_package")

        with tools.environment_append({"CONAN_SYSREQUIRES_SUDO": "True"}):

            # Chocolatey is an optional package manager on Windows
            if platform.system() == "Windows" and which("choco.exe"):
                os_info.is_solaris = False
                os_info.is_windows = True
                patched_with_apt.return_value = False

                spt = SystemPackageTool(runner=runner, os_info=os_info, output=self.out,
                                        tool=ChocolateyTool(output=self.out))
                spt.update()
                self.assertEqual(runner.command_called, "choco outdated")
                spt.install("a_package", force=True)
                self.assertEqual(runner.command_called, "choco install --yes a_package")
                spt.install("a_package", force=False)
                self.assertEqual(runner.command_called,
                                 'choco search --local-only --exact a_package | '
                                 'findstr /c:"1 packages installed."')

    def test_opensuse_zypper_aptitude(self):
        # https://github.com/conan-io/conan/issues/8737
        os_info = OSInfo()
        os_info.is_linux = True
        os_info.is_solaris = False
        os_info.is_macos = False
        os_info.is_windows = False
        os_info.linux_distro = "opensuse"
        runner = RunnerMock()

        with tools.environment_append({"CONAN_SYSREQUIRES_SUDO": "False"}):
            spt = SystemPackageTool(runner=runner, os_info=os_info, output=self.out)
            spt.update()
            self.assertEqual(runner.command_called, "zypper --non-interactive ref")

    def test_system_package_tool_try_multiple(self):
        packages = ["a_package", "another_package", "yet_another_package"]
        with tools.environment_append({"CONAN_SYSREQUIRES_SUDO": "True"}):
            runner = RunnerMultipleMock(['dpkg-query -W -f=\'${Status}\' another_package | '
                                         'grep -q "ok installed"'])
            spt = SystemPackageTool(runner=runner, tool=AptTool(output=self.out), output=self.out)
            spt.install(packages)
            self.assertEqual(2, runner.calls)
            runner = RunnerMultipleMock(["sudo -A apt-get update",
                                         "sudo -A apt-get install -y --no-install-recommends"
                                         " yet_another_package"])
            spt = SystemPackageTool(runner=runner, tool=AptTool(output=self.out), output=self.out)
            spt.install(packages)
            self.assertEqual(7, runner.calls)

            runner = RunnerMultipleMock(["sudo -A apt-get update"])
            spt = SystemPackageTool(runner=runner, tool=AptTool(output=self.out), output=self.out)
            with self.assertRaises(ConanException):
                spt.install(packages)
            self.assertEqual(7, runner.calls)

    def test_system_package_tool_mode(self):
        """
        System Package Tool mode is defined by CONAN_SYSREQUIRES_MODE env variable.
        Allowed values: (enabled, verify, disabled). Parser accepts it in lower/upper
        case or any combination.
        """
        packages = ["a_package", "another_package", "yet_another_package"]

        # Check invalid mode raises ConanException
        with tools.environment_append({
            "CONAN_SYSREQUIRES_MODE": "test_not_valid_mode",
            "CONAN_SYSREQUIRES_SUDO": "True"
        }):
            runner = RunnerMultipleMock([])
            spt = SystemPackageTool(runner=runner, tool=AptTool(output=self.out), output=self.out)
            with self.assertRaises(ConanException) as exc:
                spt.install(packages)
            self.assertIn("CONAN_SYSREQUIRES_MODE=test_not_valid_mode is not allowed",
                          str(exc.exception))
            self.assertEqual(0, runner.calls)

        # Check verify mode, a package report should be shown in output and ConanException raised.
        # No system packages are installed
        with tools.environment_append({
            "CONAN_SYSREQUIRES_MODE": "VeRiFy",
            "CONAN_SYSREQUIRES_SUDO": "True"
        }):
            packages = ["verify_package", "verify_another_package", "verify_yet_another_package"]
            runner = RunnerMultipleMock(["sudo -A apt-get update"])
            spt = SystemPackageTool(runner=runner, tool=AptTool(output=self.out), output=self.out)
            with self.assertRaises(ConanInvalidSystemRequirements) as exc:
                spt.install(packages)
            self.assertIn("Aborted due to CONAN_SYSREQUIRES_MODE=", str(exc.exception))
            self.assertIn('\n'.join(packages), self.out)
            self.assertEqual(3, runner.calls)

        # Check disabled mode, a package report should be displayed in output.
        # No system packages are installed
        with tools.environment_append({
            "CONAN_SYSREQUIRES_MODE": "DiSaBlEd",
            "CONAN_SYSREQUIRES_SUDO": "True"
        }):
            packages = ["disabled_package", "disabled_another_package",
                        "disabled_yet_another_package"]
            runner = RunnerMultipleMock(["sudo -A apt-get update"])
            spt = SystemPackageTool(runner=runner, tool=AptTool(output=self.out), output=self.out)
            spt.install(packages)
            self.assertIn('\n'.join(packages), self.out)
            self.assertEqual(0, runner.calls)

        # Check enabled, default mode, system packages must be installed.
        with tools.environment_append({
            "CONAN_SYSREQUIRES_MODE": "EnAbLeD",
            "CONAN_SYSREQUIRES_SUDO": "True"
        }):
            runner = RunnerMultipleMock(["sudo -A apt-get update"])
            spt = SystemPackageTool(runner=runner, tool=AptTool(output=self.out), output=self.out)
            with self.assertRaises(ConanException) as exc:
                spt.install(packages)
            self.assertNotIn("CONAN_SYSREQUIRES_MODE", str(exc.exception))
            self.assertEqual(7, runner.calls)

        # Check default_mode. The environment variable is not set and should behave like
        # the default_mode
        with tools.environment_append({
            "CONAN_SYSREQUIRES_MODE": None,
            "CONAN_SYSREQUIRES_SUDO": "True"
        }):
            packages = ["verify_package", "verify_another_package", "verify_yet_another_package"]
            runner = RunnerMultipleMock(["sudo -A apt-get update"])
            spt = SystemPackageTool(runner=runner, tool=AptTool(output=self.out), output=self.out,
                                    default_mode="verify")
            with self.assertRaises(ConanInvalidSystemRequirements) as exc:
                spt.install(packages)
            self.assertIn("Aborted due to CONAN_SYSREQUIRES_MODE=", str(exc.exception))
            self.assertIn('\n'.join(packages), self.out)
            self.assertEqual(3, runner.calls)

    def test_system_package_tool_installed(self):
        if (platform.system() != "Linux" and platform.system() != "Macos" and
                platform.system() != "Windows"):
            return
        if platform.system() == "Windows" and not which("choco.exe"):
            return
        spt = SystemPackageTool(output=self.out)
        expected_package = "git"
        if platform.system() == "Windows" and which("choco.exe"):
            spt = SystemPackageTool(tool=ChocolateyTool(output=self.out), output=self.out)
            # Git is not installed by default on Chocolatey
            expected_package = "chocolatey"
        else:
            if platform.system() != "Windows" and not which("git"):
                return
        # The expected should be installed on development/testing machines
        self.assertTrue(spt._tool.installed(expected_package))
        self.assertTrue(spt.installed(expected_package))
        # This package hopefully doesn't exist
        self.assertFalse(spt._tool.installed("oidfjgesiouhrgioeurhgielurhgaeiorhgioearhgoaeirhg"))
        self.assertFalse(spt.installed("oidfjgesiouhrgioeurhgielurhgaeiorhgioearhgoaeirhg"))

    def test_system_package_tool_fail_when_not_0_returned(self):
        def get_linux_error_message():
            """
            Get error message for Linux platform if distro is supported, None otherwise
            """
            os_info = OSInfo()
            update_command = None
            if os_info.with_apt:
                update_command = "sudo -A apt-get update"
            elif os_info.with_yum:
                update_command = "sudo -A yum check-update -y"
            elif os_info.with_dnf:
                update_command = "sudo -A dnf check-update -y"
            elif os_info.with_zypper:
                update_command = "sudo -A zypper --non-interactive ref"
            elif os_info.with_pacman:
                update_command = "sudo -A pacman -Syyu --noconfirm"

            return ("Command '{0}' failed".format(update_command)
                    if update_command is not None else None)

        platform_update_error_msg = {
            "Linux": get_linux_error_message(),
            "Darwin": "Command 'brew update' failed",
            "Windows": "Command 'choco outdated' failed" if which("choco.exe") else None,
        }

        runner = RunnerMock(return_ok=False)
        output = ConanOutput(StringIO())
        pkg_tool = ChocolateyTool(output=output) if which("choco.exe") else None
        spt = SystemPackageTool(runner=runner, tool=pkg_tool, output=output)

        msg = platform_update_error_msg.get(platform.system(), None)
        if msg is not None:
            with six.assertRaisesRegex(self, ConanException, msg):
                spt.update()
        else:
            spt.update()  # Won't raise anything because won't do anything

    def test_install_all_packages(self):
        """ SystemPackageTool must install all packages
        """
        # No packages installed
        packages = ["a_package", "another_package", "yet_another_package"]
        with tools.environment_append({"CONAN_SYSREQUIRES_SUDO": "True"}):
            runner = RunnerMultipleMock(["sudo -A apt-get update",
                                         "sudo -A apt-get install -y --no-install-recommends"
                                         " a_package another_package yet_another_package",
                                         ])
            spt = SystemPackageTool(runner=runner, tool=AptTool(output=self.out), output=self.out)
            spt.install_packages(packages)
            self.assertEqual(5, runner.calls)

    def test_install_few_packages(self):
        """ SystemPackageTool must install 2 packages only
        """
        packages = ["a_package", "another_package", "yet_another_package"]
        with tools.environment_append({"CONAN_SYSREQUIRES_SUDO": "True"}):
            runner = RunnerMultipleMock(['dpkg-query -W -f=\'${Status}\' a_package | '
                                         'grep -q "ok installed"',
                                         "sudo -A apt-get update",
                                         "sudo -A apt-get install -y --no-install-recommends"
                                         " another_package yet_another_package",
                                         ])
            spt = SystemPackageTool(runner=runner, tool=AptTool(output=self.out), output=self.out)
            spt.install_packages(packages)
            self.assertEqual(5, runner.calls)

    def test_packages_installed(self):
        """ SystemPackageTool must not install. All packages are installed.
        """
        packages = ["a_package", "another_package", "yet_another_package"]
        with tools.environment_append({"CONAN_SYSREQUIRES_SUDO": "True"}):
            runner = RunnerMultipleMock(['dpkg-query -W -f=\'${Status}\' a_package | '
                                         'grep -q "ok installed"',
                                         'dpkg-query -W -f=\'${Status}\' another_package | '
                                         'grep -q "ok installed"',
                                         'dpkg-query -W -f=\'${Status}\' yet_another_package | '
                                         'grep -q "ok installed"',
                                         ])
            spt = SystemPackageTool(runner=runner, tool=AptTool(output=self.out), output=self.out)
            spt.install_packages(packages)
            self.assertEqual(3, runner.calls)

    def test_empty_package_list(self):
        """ Install nothing
        """
        packages = []
        with tools.environment_append({"CONAN_SYSREQUIRES_SUDO": "True"}):
            runner = RunnerMultipleMock()
            spt = SystemPackageTool(runner=runner, tool=AptTool(output=self.out),
                                    output=self.out)
            spt.install_packages(packages)
            self.assertEqual(0, runner.calls)

    def test_install_variants_and_packages(self):
        """ SystemPackageTool must install one of variants and all packages at same list
        """
        packages = [("varianta", "variantb", "variantc"), "a_package", "another_package"]
        with tools.environment_append({"CONAN_SYSREQUIRES_SUDO": "True"}):
            runner = RunnerMultipleMock(["sudo -A apt-get update",
                                         "sudo -A apt-get install -y --no-install-recommends"
                                         " varianta",
                                         "sudo -A apt-get update",
                                         "sudo -A apt-get install -y --no-install-recommends"
                                         " a_package another_package",
                                         ])
            spt = SystemPackageTool(runner=runner, tool=AptTool(output=self.out), output=self.out)
            spt.install_packages(packages)
            self.assertEqual(8, runner.calls)

    def test_installed_variant_and_install_packages(self):
        """ Only packages must be installed. Variants are already installed
        """
        packages = [("varianta", "variantb", "variantc"), "a_package", "another_package"]
        with tools.environment_append({"CONAN_SYSREQUIRES_SUDO": "True"}):
            runner = RunnerMultipleMock(['dpkg-query -W -f=\'${Status}\' varianta | '
                                         'grep -q "ok installed"',
                                         "sudo -A apt-get update",
                                         "sudo -A apt-get install -y --no-install-recommends"
                                         " a_package another_package",
                                         ])
            spt = SystemPackageTool(runner=runner, tool=AptTool(output=self.out), output=self.out)
            spt.install_packages(packages)
            self.assertEqual(5, runner.calls)

    def test_installed_packages_and_install_variant(self):
        """ Only variant must be installed. Packages are already installed
        """
        packages = [("varianta", "variantb", "variantc"), "a_package", "another_package"]
        with tools.environment_append({"CONAN_SYSREQUIRES_SUDO": "True"}):
            runner = RunnerMultipleMock(['dpkg-query -W -f=\'${Status}\' a_package | '
                                         'grep -q "ok installed"',
                                         'dpkg-query -W -f=\'${Status}\' another_package | '
                                         'grep -q "ok installed"',
                                         "sudo -A apt-get update",
                                         "sudo -A apt-get install -y --no-install-recommends"
                                         " varianta",
                                         ])
            spt = SystemPackageTool(runner=runner, tool=AptTool(output=self.out), output=self.out)
            spt.install_packages(packages)
            self.assertEqual(7, runner.calls)

    def test_variants_and_packages_installed(self):
        """ Install nothing, all is already installed
        """
        packages = [("varianta", "variantb", "variantc"), "a_package", "another_package"]
        with tools.environment_append({"CONAN_SYSREQUIRES_SUDO": "True"}):
            runner = RunnerMultipleMock(['dpkg-query -W -f=\'${Status}\' varianta | '
                                         'grep -q "ok installed"',
                                         'dpkg-query -W -f=\'${Status}\' a_package | '
                                         'grep -q "ok installed"',
                                         'dpkg-query -W -f=\'${Status}\' another_package | '
                                         'grep -q "ok installed"',
                                         ])
            spt = SystemPackageTool(runner=runner, tool=AptTool(output=self.out), output=self.out)
            spt.install_packages(packages)
            self.assertEqual(3, runner.calls)

    def test_empty_variants_and_packages(self):
        packages = [(),]
        with tools.environment_append({"CONAN_SYSREQUIRES_SUDO": "True"}):
            runner = RunnerMultipleMock()
            spt = SystemPackageTool(runner=runner, tool=AptTool(output=self.out),
                                    output=self.out)
            spt.install_packages(packages)
            self.assertEqual(0, runner.calls)

    def test_install_all_multiple_package_list(self):
        """ Separated string list must be treated as full package list to be installed
        """
        packages = "varianta variantb", "variantc variantd"
        runner = RunnerMultipleMock([])
        spt = SystemPackageTool(runner=runner, tool=AptTool(output=self.out), output=self.out)
        with self.assertRaises(ConanException) as error:
            spt.install_packages(packages)
            self.assertEqual("Each string must contain only one package to be installed."
                             " Use a list instead e.g. ['foo', 'bar'].", str(error.exception))



import textwrap
import unittest

from conans.test.utils.tools import TestClient


class InitTest(unittest.TestCase):
    def test_wrong_init(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class Lib(ConanFile):
                def init(self):
                    random_error
            """)

        client.save({"conanfile.py": conanfile})
        client.run("inspect .", assert_error=True)
        self.assertIn("Error in init() method, line 5", client.out)
        self.assertIn("name 'random_error' is not defined", client.out)

    def test_init(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conans import ConanFile, load
            import os
            import json
            class Lib(ConanFile):
                exports = "data.json"
                def init(self):
                    data = load(os.path.join(self.recipe_folder, "data.json"))
                    d = json.loads(data)
                    self.license = d["license"]
                    self.description = d["description"]
                def build(self):
                    self.output.info("LICENSE: %s" % self.license)
            """)
        data = '{"license": "MIT", "description": "MyDescription"}'
        client.save({"conanfile.py": conanfile,
                     "data.json": data})

        client.run("inspect .")
        self.assertIn("description: MyDescription", client.out)
        self.assertIn("license: MIT", client.out)
        client.run("create . pkg/0.1@user/testing")
        client.run("inspect pkg/0.1@user/testing")
        self.assertIn("description: MyDescription", client.out)
        self.assertIn("license: MIT", client.out)



import textwrap

from conans.client.graph.graph import CONTEXT_BUILD, CONTEXT_HOST
from conans.model.profile import Profile
from conans.model.ref import ConanFileReference
from conans.test.integration.graph.core.cross_build._base_test_case import CrossBuildingBaseTestCase


class ProtobufTest(CrossBuildingBaseTestCase):
    """ Application that requires 'protobuf' as a regular requires and as a 'build_requires'.
        The package 'protobuf' requires 'zlib' in different versions depending on the context.

    """

    protobuf = textwrap.dedent("""
        from conans import ConanFile

        class Protobuf(ConanFile):
            settings = "os"
            def requirements(self):
                if self.settings.os == "Host":
                    self.requires("zlib/1.0@user/channel")
                else:
                    self.requires("zlib/2.0@user/channel")

            def build(self):
                self.output.info(">> settings.os:".format(self.settings.os))
                self.output.info("ZLIB: %s" % self.deps_cpp_info["zlib"].libs)

            def package_info(self):
                protobuf_str = "protobuf-host" if self.settings.os == "Host" else "protobuf-build"

                self.cpp_info.includedirs = [protobuf_str, ]
                self.cpp_info.libdirs = [protobuf_str, ]
                self.cpp_info.bindirs = [protobuf_str, ]
                self.cpp_info.libs = [protobuf_str]

                self.env_info.PATH.append(protobuf_str)
                self.env_info.OTHERVAR = protobuf_str
        """)

    app = textwrap.dedent("""
        from conans import ConanFile

        class App(ConanFile):
            settings = "os"
            requires = "protobuf/testing@user/channel"
            build_requires = "protobuf/testing@user/channel"

            def build(self):
                self.output.info(">> settings.os:".format(self.settings.os))
                self.output.info("ZLIB: %s" % self.deps_cpp_info["zlib"].libs)
        """)

    cmake = textwrap.dedent("""
        from conans import ConanFile

        class CMake(ConanFile):
            settings = "os"
            requires = "bzip/3.0@user/channel"

            def build(self):
                self.output.info(">> settings.os:".format(self.settings.os))
                self.output.info("BZIP: %s" % self.deps_cpp_info["bzip"].libs)

            def package_info(self):
                assert self.settings.os == "Build"

                self.env_info.PATH.append("cmake_build")
                self.env_info.OTHERVAR = "cmake_build"
        """)

    zlib = CrossBuildingBaseTestCase.library_tpl.render(name="zlib")
    bzip = CrossBuildingBaseTestCase.library_tpl.render(name="bzip")

    zlib1_ref = ConanFileReference.loads("zlib/1.0@user/channel")
    zlib2_ref = ConanFileReference.loads("zlib/2.0@user/channel")
    bzip_ref = ConanFileReference.loads("bzip/3.0@user/channel")

    def setUp(self):
        super(ProtobufTest, self).setUp()
        self._cache_recipe(self.zlib1_ref, self.zlib)
        self._cache_recipe(self.zlib2_ref, self.zlib)
        self._cache_recipe(self.bzip_ref, self.bzip)
        self._cache_recipe(self.cmake_ref, self.cmake)
        self._cache_recipe(self.protobuf_ref, self.protobuf)
        self._cache_recipe(self.app_ref, self.app)

    def test_crossbuilding(self):
        profile_host = Profile()
        profile_host.settings["os"] = "Host"
        profile_host.process_settings(self.cache)

        profile_build = Profile()
        profile_build.settings["os"] = "Build"
        profile_build.process_settings(self.cache)

        deps_graph = self._build_graph(profile_host=profile_host, profile_build=profile_build, install=True)

        # Check HOST packages
        #   - app
        app = deps_graph.root.dependencies[0].dst
        self.assertEqual(len(app.dependencies), 2)
        self.assertEqual(app.conanfile.name, "app")
        self.assertEqual(app.context, CONTEXT_HOST)
        self.assertEqual(app.conanfile.settings.os, profile_host.settings['os'])

        #   - app::deps_cpp_info
        protobuf_host_cpp_info = app.conanfile.deps_cpp_info["protobuf"]
        self.assertEqual(protobuf_host_cpp_info.includedirs, ['protobuf-host'])
        self.assertEqual(protobuf_host_cpp_info.libdirs, ['protobuf-host'])
        self.assertEqual(protobuf_host_cpp_info.bindirs, ['protobuf-host'])
        protobuf_host_cpp_info = app.conanfile.deps_cpp_info["zlib"]
        self.assertEqual(protobuf_host_cpp_info.libs, ['zlib-host-1.0'])
        self.assertListEqual(list(app.conanfile.deps_cpp_info.libs), ['protobuf-host', 'zlib-host-1.0'])

        #   - app::deps_env_info
        protobuf_env_info = app.conanfile.deps_env_info["protobuf"]
        self.assertEqual(protobuf_env_info.PATH, ['protobuf-build'])
        self.assertEqual(protobuf_env_info.OTHERVAR, 'protobuf-build')
        zlib_env_info = app.conanfile.deps_env_info["zlib"]
        self.assertEqual(zlib_env_info.vars["PATH"], ['zlib-build-2.0'])
        self.assertEqual(app.conanfile.deps_env_info.vars["PATH"], ['protobuf-build', 'zlib-build-2.0'])

        #   - protobuf
        protobuf_host = app.dependencies[0].dst
        self.assertEqual(protobuf_host.conanfile.name, "protobuf")
        self.assertEqual(protobuf_host.context, CONTEXT_HOST)
        self.assertEqual(protobuf_host.conanfile.settings.os, profile_host.settings['os'])

        #   - protobuf::deps_cpp_info
        zlib_cpp_info = protobuf_host.conanfile.deps_cpp_info["zlib"]
        self.assertEqual(zlib_cpp_info.libs, ['zlib-host-1.0'])

        #   - protobuf::deps_env_info (no zlib)
        zlib_env_info = list(protobuf_host.conanfile.deps_env_info.deps)
        self.assertEqual(zlib_env_info, [])

        # Check BUILD packages
        #   - protobuf
        protobuf_build = app.dependencies[1].dst
        self.assertEqual(protobuf_build.conanfile.name, "protobuf")
        self.assertEqual(protobuf_build.context, CONTEXT_BUILD)
        self.assertEqual(protobuf_build.conanfile.settings.os, profile_build.settings['os'])

        #   - protobuf::deps_cpp_info
        zlib_cpp_info = protobuf_build.conanfile.deps_cpp_info["zlib"]
        self.assertEqual(zlib_cpp_info.libs, ['zlib-build-2.0'])

        #   - protobuf::deps_env_info (no zlib)
        zlib_env_info = list(protobuf_build.conanfile.deps_env_info.deps)
        self.assertEqual(zlib_env_info, [])

    def test_crossbuilding_with_build_require_cmake(self):
        profile_host = Profile()
        profile_host.settings["os"] = "Host"
        profile_host.process_settings(self.cache)
        profile_host.build_requires["*"] = [self.cmake_ref]

        profile_build = Profile()
        profile_build.settings["os"] = "Build"
        profile_build.process_settings(self.cache)
        profile_build.build_requires["*"] = [self.cmake_ref]
        profile_build.build_requires["zlib/3.0*"] = []

        deps_graph = self._build_graph(profile_host=profile_host, profile_build=profile_build, install=True)

        # Check HOST packages
        #   - app
        app = deps_graph.root.dependencies[0].dst
        self.assertEqual(len(app.dependencies), 3)
        self.assertEqual(app.conanfile.name, "app")
        self.assertEqual(app.context, CONTEXT_HOST)
        self.assertEqual(app.conanfile.settings.os, profile_host.settings['os'])

        #   - app::deps_cpp_info
        protobuf_host_cpp_info = app.conanfile.deps_cpp_info["protobuf"]
        self.assertEqual(protobuf_host_cpp_info.includedirs, ['protobuf-host'])
        self.assertEqual(protobuf_host_cpp_info.libdirs, ['protobuf-host'])
        self.assertEqual(protobuf_host_cpp_info.bindirs, ['protobuf-host'])
        zlib_host_cpp_info = app.conanfile.deps_cpp_info["zlib"]
        self.assertEqual(zlib_host_cpp_info.libs, ['zlib-host-1.0'])
        self.assertListEqual(list(app.conanfile.deps_cpp_info.libs), ['protobuf-host', 'zlib-host-1.0'])

        #   - app::deps_env_info
        protobuf_env_info = app.conanfile.deps_env_info["protobuf"]
        self.assertEqual(protobuf_env_info.PATH, ['protobuf-build'])
        self.assertEqual(protobuf_env_info.OTHERVAR, 'protobuf-build')
        zlib_env_info = app.conanfile.deps_env_info["zlib"]
        self.assertEqual(zlib_env_info.vars["PATH"], ['zlib-build-2.0'])
        self.assertEqual(app.conanfile.deps_env_info.vars["PATH"],
                         ['cmake_build', 'protobuf-build', 'bzip-build-3.0', 'zlib-build-2.0'])

        #   - protobuf
        protobuf_host = app.dependencies[0].dst
        self.assertEqual(protobuf_host.conanfile.name, "protobuf")
        self.assertEqual(protobuf_host.context, CONTEXT_HOST)
        self.assertEqual(protobuf_host.conanfile.settings.os, profile_host.settings['os'])

        #   - protobuf::deps_cpp_info
        zlib_cpp_info = protobuf_host.conanfile.deps_cpp_info["zlib"]
        self.assertEqual(zlib_cpp_info.libs, ['zlib-host-1.0'])
        self.assertListEqual(list(protobuf_host.conanfile.deps_cpp_info.libs), ['zlib-host-1.0'])

        #   - protobuf::deps_env_info (no zlib)
        zlib_env_info = list(protobuf_host.conanfile.deps_env_info.deps)
        self.assertEqual(zlib_env_info, ['cmake', 'bzip'])
        self.assertEqual(protobuf_host.conanfile.deps_env_info.vars["PATH"], ['cmake_build', 'bzip-build-3.0'])

        #   - zlib
        zlib_host = protobuf_host.dependencies[0].dst
        self.assertEqual(zlib_host.conanfile.name, "zlib")
        self.assertEqual(zlib_host.context, CONTEXT_HOST)
        self.assertEqual(zlib_host.conanfile.settings.os, profile_host.settings['os'])
        self.assertListEqual(list(zlib_host.conanfile.deps_cpp_info.libs), [])
        self.assertEqual(zlib_host.conanfile.deps_env_info.vars["PATH"], ['cmake_build', 'bzip-build-3.0'])

        # Check BUILD packages
        #   - protobuf
        protobuf_build = app.dependencies[1].dst
        self.assertEqual(protobuf_build.conanfile.name, "protobuf")
        self.assertEqual(protobuf_build.context, CONTEXT_BUILD)
        self.assertEqual(protobuf_build.conanfile.settings.os, profile_build.settings['os'])

        #   - protobuf::deps_cpp_info
        zlib_cpp_info = protobuf_build.conanfile.deps_cpp_info["zlib"]
        self.assertListEqual(list(zlib_cpp_info.libs), ['zlib-build-2.0'])
        self.assertListEqual(list(protobuf_build.conanfile.deps_cpp_info.libs), ['zlib-build-2.0'])

        #   - protobuf::deps_env_info (no zlib)
        zlib_env_info = list(protobuf_build.conanfile.deps_env_info.deps)
        self.assertEqual(zlib_env_info, ['cmake', 'bzip'])
        self.assertEqual(protobuf_build.conanfile.deps_env_info.vars["PATH"], ['cmake_build', 'bzip-build-3.0'])



import textwrap
import unittest

import mock

from conans import __version__
from conans.test.utils.tools import TestClient


class RequiredConanVersionTest(unittest.TestCase):

    def test_required_conan_version(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conans import ConanFile

            required_conan_version = ">=100.0"

            class Lib(ConanFile):
                pass
            """)
        client.save({"conanfile.py": conanfile})
        client.run("export . pkg/1.0@", assert_error=True)
        self.assertIn("Current Conan version (%s) does not satisfy the defined one (>=100.0)"
                      % __version__, client.out)
        client.run("inspect . ", assert_error=True)
        self.assertIn("Current Conan version (%s) does not satisfy the defined one (>=100.0)"
                      % __version__, client.out)
        with mock.patch("conans.client.conf.required_version.client_version", "101.0"):
            client.run("export . pkg/1.0@")

        with mock.patch("conans.client.conf.required_version.client_version", "101.0-dev"):
            client.run("export . pkg/1.0@")

        client.run("install pkg/1.0@", assert_error=True)
        self.assertIn("Current Conan version (%s) does not satisfy the defined one (>=100.0)"
                      % __version__, client.out)

    def test_required_conan_version_with_loading_issues(self):
        # https://github.com/conan-io/conan/issues/11239
        client = TestClient()
        conanfile = textwrap.dedent("""
                    from conan import missing_import

                    required_conan_version = ">=100.0"

                    class Lib(ConanFile):
                        pass
                    """)
        client.save({"conanfile.py": conanfile})
        client.run("export . pkg/1.0@", assert_error=True)
        self.assertIn("Current Conan version (%s) does not satisfy the defined one (>=100.0)"
                      % __version__, client.out)

        # Assigning required_conan_version without spaces
        conanfile = textwrap.dedent("""
                            from conan import missing_import

                            required_conan_version=">=100.0"

                            class Lib(ConanFile):
                                pass
                            """)
        client.save({"conanfile.py": conanfile})
        client.run("export . pkg/1.0@", assert_error=True)
        self.assertIn("Current Conan version (%s) does not satisfy the defined one (>=100.0)"
                      % __version__, client.out)

        # If the range is correct, everything works, of course
        conanfile = textwrap.dedent("""
                            from conan import ConanFile

                            required_conan_version = ">1.0.0"

                            class Lib(ConanFile):
                                pass
                            """)
        client.save({"conanfile.py": conanfile})
        client.run("export . pkg/1.0@")



import os
import platform
import textwrap
import time
import re

import pytest

from conan.tools.env.environment import environment_wrap_command
from conans.model.ref import ConanFileReference, PackageReference
from conans.test.assets.autotools import gen_makefile_am, gen_configure_ac, gen_makefile
from conans.test.assets.sources import gen_function_cpp
from conans.test.functional.utils import check_exe_run
from conans.test.utils.tools import TestClient, TurboTestClient
from conans.util.files import touch


@pytest.mark.skipif(platform.system() not in ["Linux", "Darwin"], reason="Requires Autotools")
@pytest.mark.tool_autotools()
def test_autotools():
    client = TestClient(path_with_spaces=False)
    client.run("new hello/0.1 --template=cmake_lib")
    client.run("create .")

    main = gen_function_cpp(name="main", includes=["hello"], calls=["hello"])
    makefile_am = gen_makefile_am(main="main", main_srcs="main.cpp")
    configure_ac = gen_configure_ac()

    conanfile = textwrap.dedent("""
        from conans import ConanFile
        from conan.tools.gnu import Autotools

        class TestConan(ConanFile):
            requires = "hello/0.1"
            settings = "os", "compiler", "arch", "build_type"
            exports_sources = "configure.ac", "Makefile.am", "main.cpp"
            generators = "AutotoolsDeps", "AutotoolsToolchain"

            def build(self):
                self.run("aclocal")
                self.run("autoconf")
                self.run("automake --add-missing --foreign")
                autotools = Autotools(self)
                autotools.configure()
                autotools.make()
        """)

    client.save({"conanfile.py": conanfile,
                 "configure.ac": configure_ac,
                 "Makefile.am": makefile_am,
                 "main.cpp": main}, clean_first=True)
    client.run("install .")
    client.run("build .")
    client.run_command("./main")
    cxx11_abi = 0 if platform.system() == "Linux" else None
    compiler = "gcc" if platform.system() == "Linux" else "apple-clang"
    host_arch = client.get_default_host_profile().settings['arch']
    check_exe_run(client.out, "main", compiler, None, "Release", host_arch, None, cxx11_abi=cxx11_abi)
    assert "hello/0.1: Hello World Release!" in client.out


def build_windows_subsystem(profile, make_program, subsystem):
    """ The AutotoolsDeps can be used also in pure Makefiles, if the makefiles follow
    the Autotools conventions
    """
    # FIXME: cygwin in CI (my local machine works) seems broken for path with spaces
    client = TestClient(path_with_spaces=False)
    client.run("new hello/0.1 --template=cmake_lib")
    # TODO: Test Windows subsystems in CMake, at least msys is broken
    os.rename(os.path.join(client.current_folder, "test_package"),
              os.path.join(client.current_folder, "test_package2"))
    client.save({"profile": profile})
    client.run("create . --profile=profile")

    main = gen_function_cpp(name="main", includes=["hello"], calls=["hello"])
    makefile = gen_makefile(apps=["app"])

    conanfile = textwrap.dedent("""
        from conans import ConanFile
        from conan.tools.gnu import AutotoolsToolchain, Autotools, AutotoolsDeps

        class TestConan(ConanFile):
            requires = "hello/0.1"
            settings = "os", "compiler", "arch", "build_type"
            exports_sources = "Makefile"
            generators = "AutotoolsDeps", "AutotoolsToolchain"

            def build(self):
                autotools = Autotools(self)
                autotools.make()
        """)
    client.save({"app.cpp": main,
                 "Makefile": makefile,
                 "conanfile.py": conanfile,
                 "profile": profile}, clean_first=True)

    client.run("install . --profile=profile")
    cmd = environment_wrap_command(["conanbuildenv",
                                    "conanautotoolstoolchain",
                                    "conanautotoolsdeps"], client.current_folder, make_program)
    client.run_command(cmd)
    client.run_command("app")
    # TODO: fill compiler version when ready
    check_exe_run(client.out, "main", "gcc", None, "Release", "x86_64", None, subsystem=subsystem)
    assert "hello/0.1: Hello World Release!" in client.out

    client.save({"app.cpp": gen_function_cpp(name="main", msg="main2",
                                             includes=["hello"], calls=["hello"])})
    # Make sure it is newer
    t = time.time() + 1
    touch(os.path.join(client.current_folder, "app.cpp"), (t, t))

    client.run("build .")
    client.run_command("app")
    # TODO: fill compiler version when ready
    check_exe_run(client.out, "main2", "gcc", None, "Release", "x86_64", None, cxx11_abi=0,
                  subsystem=subsystem)
    assert "hello/0.1: Hello World Release!" in client.out
    return client.out


@pytest.mark.tool_cygwin
@pytest.mark.skipif(platform.system() != "Windows", reason="Needs windows")
def test_autotoolsdeps_cygwin():
    gcc = textwrap.dedent("""
        [settings]
        os=Windows
        os.subsystem=cygwin
        compiler=gcc
        compiler.version=4.9
        compiler.libcxx=libstdc++
        arch=x86_64
        build_type=Release
        """)
    build_windows_subsystem(gcc, make_program="make", subsystem="cygwin")


@pytest.mark.tool_mingw64
@pytest.mark.skipif(platform.system() != "Windows", reason="Needs windows")
def test_autotoolsdeps_mingw_msys():
    gcc = textwrap.dedent("""
        [settings]
        os=Windows
        compiler=gcc
        compiler.version=4.9
        compiler.libcxx=libstdc++
        arch=x86_64
        build_type=Release
        """)
    build_windows_subsystem(gcc, make_program="mingw32-make", subsystem="mingw64")


@pytest.mark.tool_msys2
@pytest.mark.skipif(platform.system() != "Windows", reason="Needs windows")
# If we use the cmake inside msys2, it fails, so better force our own cmake
@pytest.mark.tool_cmake
def test_autotoolsdeps_msys():
    gcc = textwrap.dedent("""
        [settings]
        os=Windows
        os.subsystem=msys2
        compiler=gcc
        compiler.version=4.9
        compiler.libcxx=libstdc++
        arch=x86_64
        build_type=Release
        """)
    build_windows_subsystem(gcc, make_program="make", subsystem="msys2")


@pytest.mark.skipif(platform.system() not in ["Linux", "Darwin"], reason="Requires Autotools")
@pytest.mark.tool_autotools()
def test_install_output_directories():
    """
    If we change the libdirs of the cpp.package, as we are doing cmake.install, the output directory
    for the libraries is changed
    """
    client = TurboTestClient(path_with_spaces=False)
    client.run("new hello/1.0 --template cmake_lib")
    client.run("create .")
    consumer_conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.gnu import Autotools

        class TestConan(ConanFile):
            requires = "hello/1.0"
            settings = "os", "compiler", "arch", "build_type"
            exports_sources = "configure.ac", "Makefile.am", "main.cpp", "consumer.h"
            generators = "AutotoolsDeps", "AutotoolsToolchain"

            def layout(self):
                self.folders.build = "."
                self.cpp.package.bindirs = ["mybin"]

            def build(self):
                self.run("aclocal")
                self.run("autoconf")
                self.run("automake --add-missing --foreign")
                autotools = Autotools(self)
                autotools.configure()
                autotools.make()
                autotools.install()
    """)

    main = gen_function_cpp(name="main", includes=["hello"], calls=["hello"])
    makefile_am = gen_makefile_am(main="main", main_srcs="main.cpp")
    configure_ac = gen_configure_ac()
    client.save({"conanfile.py": consumer_conanfile,
                 "configure.ac": configure_ac,
                 "Makefile.am": makefile_am,
                 "main.cpp": main}, clean_first=True)
    ref = ConanFileReference.loads("zlib/1.2.11")
    pref = client.create(ref, conanfile=consumer_conanfile)
    p_folder = client.cache.package_layout(pref.ref).package(pref)
    assert os.path.exists(os.path.join(p_folder, "mybin", "main"))
    assert not os.path.exists(os.path.join(p_folder, "bin"))


@pytest.mark.skipif(platform.system() not in ["Linux", "Darwin"], reason="Requires Autotools")
@pytest.mark.tool_autotools()
def test_autotools_with_pkgconfigdeps():
    client = TestClient(path_with_spaces=False)
    client.run("new hello/1.0 --template cmake_lib")
    client.run("create .")

    consumer_conanfile = textwrap.dedent("""
        [requires]
        hello/1.0
        [generators]
        AutotoolsToolchain
        PkgConfigDeps
    """)
    client.save({"conanfile.txt": consumer_conanfile}, clean_first=True)
    client.run("install .")

    client.run_command(". ./conanautotoolstoolchain.sh && "
                       "pkg-config --cflags hello && "
                       "pkg-config --libs-only-l hello && "
                       "pkg-config --libs-only-L --libs-only-other hello")

    assert re.search("I.*hello.*1.0.*include", str(client.out))
    assert "-lhello" in client.out
    assert re.search("L.*hello.*1.0.*package", str(client.out))


@pytest.mark.skipif(platform.system() not in ["Linux", "Darwin"], reason="Requires Autotools")
@pytest.mark.tool_autotools()
def test_autotools_option_checking():
    # https://github.com/conan-io/conan/issues/11265
    client = TestClient(path_with_spaces=False)
    client.run("new mylib/1.0@ -m autotools_lib")
    conanfile = textwrap.dedent("""
        import os

        from conan import ConanFile
        from conan.tools.gnu import AutotoolsToolchain, Autotools
        from conan.tools.layout import basic_layout
        from conan.tools.build import cross_building
        from conan.tools.files import chdir


        class MylibTestConan(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            # VirtualBuildEnv and VirtualRunEnv can be avoided if "tools.env.virtualenv:auto_use" is defined
            # (it will be defined in Conan 2.0)
            generators = "AutotoolsDeps", "VirtualBuildEnv", "VirtualRunEnv"
            apply_env = False
            test_type = "explicit"

            def requirements(self):
                self.requires(self.tested_reference_str)

            def generate(self):
                at_toolchain = AutotoolsToolchain(self)
                # we override the default shared/static flags here
                at_toolchain.configure_args = ['--enable-option-checking=fatal']
                at_toolchain.generate()

            def build(self):
                autotools = Autotools(self)
                autotools.autoreconf()
                autotools.configure()
                autotools.make()

            def layout(self):
                basic_layout(self)

            def test(self):
                if not cross_building(self):
                    cmd = os.path.join(self.cpp.build.bindirs[0], "main")
                    self.run(cmd, env="conanrun")
            """)

    client.save({"test_package/conanfile.py": conanfile})
    client.run("create . -tf=None")

    # check that the shared flags are not added to the exe's configure, making it fail
    client.run("test test_package mylib/1.0@")
    assert "configure: error: unrecognized options: --disable-shared, --enable-static, --with-pic" \
           not in client.out


@pytest.mark.skipif(platform.system() not in ["Linux", "Darwin"], reason="Requires Autotools")
@pytest.mark.tool_autotools()
def test_autotools_arguments_override():
    client = TestClient(path_with_spaces=False)
    client.run("new mylib/1.0@ -m autotools_lib")
    conanfile = textwrap.dedent("""
        import os

        from conan import ConanFile
        from conan.tools.gnu import AutotoolsToolchain, Autotools
        from conan.tools.layout import basic_layout


        class MyLibConan(ConanFile):
            name = "mylib"
            version = "1.0"

            # Binary configuration
            settings = "os", "compiler", "build_type", "arch"

            exports_sources = "configure.ac", "Makefile.am", "src/*"

            def config_options(self):
                if self.settings.os == "Windows":
                    del self.options.fPIC

            def layout(self):
                basic_layout(self)

            def generate(self):
                at_toolchain = AutotoolsToolchain(self)
                at_toolchain.configure_args = ['--disable-shared']
                at_toolchain.make_args = ['--warn-undefined-variables']
                at_toolchain.autoreconf_args = ['--verbose']
                at_toolchain.generate()

            def build(self):
                autotools = Autotools(self)
                autotools.autoreconf(args=['--install'])
                autotools.configure(args=['--prefix=/', '--libdir=${prefix}/customlibfolder',
                                          '--includedir=${prefix}/customincludefolder',
                                          '--pdfdir=${prefix}/res'])
                autotools.make(args=['--keep-going'])

            def package(self):
                autotools = Autotools(self)
                autotools.install(args=['DESTDIR={}/somefolder'.format(self.package_folder)])

            def package_info(self):
                self.cpp_info.libs = ["mylib"]
                self.cpp_info.libdirs = ["somefolder/customlibfolder"]
                self.cpp_info.includedirs = ["somefolder/customincludefolder"]
        """)
    client.run("config set log.print_run_commands=1")
    client.save({"conanfile.py": conanfile})
    client.run("create . -tf=None")

    # autoreconf args --force that is default should not be there
    assert "--force" not in client.out
    assert "--install" in client.out

    package_id = re.search(r"mylib\/1.0: Package (\S+)", str(client.out)).group(1).replace("'", "")
    pref = PackageReference(ConanFileReference.loads("mylib/1.0"), package_id)
    package_folder = client.cache.package_layout(pref.ref).package(pref)

    # we override the default DESTDIR in the install
    assert 'DESTDIR={} '.format(package_folder) not in client.out
    assert 'DESTDIR={}/somefolder '.format(package_folder) in client.out

    # we did override the default install args
    for arg in ['--bindir=${prefix}/bin', '--sbindir=${prefix}/bin',
                '--libdir=${prefix}/lib', '--includedir=${prefix}/include',
                '--oldincludedir=${prefix}/include', '--datarootdir=${prefix}/res']:
        assert arg not in client.out

    # and use our custom arguments
    for arg in ['--prefix=/', '--libdir=${prefix}/customlibfolder',
                '--includedir=${prefix}/customincludefolder', '--pdfdir=${prefix}/res']:
        assert arg in client.out

    # check the other arguments we set are there
    assert "--disable-shared" in client.out
    assert "--warn-undefined-variables" in client.out
    assert "--verbose" in client.out
    assert "--keep-going" in client.out

    client.run("test test_package mylib/1.0@")
    assert "mylib/1.0: Hello World Release!" in client.out
